package uk.co.trinitylogic.skyfall.controller;

import java.util.ArrayList;
import java.util.Map;

import javax.validation.Valid;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import uk.co.trinitylogic.skyfall.form.Advice;
import uk.co.trinitylogic.skyfall.service.AdviceService;
import uk.co.trinitylogic.skyfall.service.ServiceService;

@Controller
public class AdviceController {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private AdviceService adviceService;

	@Autowired
	private ServiceService serviceService;

	@RequestMapping("/list-advice")
	public String listAdvice(Map<String, Object> map) {
		logger.info("go to list-advice-success");
		map.put("adviceList", adviceService.listAdvice());
		return "list-advice-success";
	}

	@RequestMapping("/search-advice-submit")
	public String searchAdvice(Map<String, Object> map, @RequestParam("searchAdvice") String searchAdvice) {
		logger.info("go to search-advice");
		try {
			map.put("searchAdviceList", adviceService.searchForAdvices((searchAdvice)));
		} catch (InterruptedException e) {
			map.put("searchAdviceList", new ArrayList<Advice>());
		}
		return "search-advice";
	}

	@RequestMapping("/add-advice")
	public String addAdvice(Map<String, Object> map) {
		map.put("serviceList", serviceService.listService());
		map.put("advice", new Advice());
		logger.info("go to add-advice");
		return "add-advice";
	}

	@RequestMapping(value = "/add-advice-submit", method = RequestMethod.POST)
	public String addAdviceSubmit(@ModelAttribute @Valid Advice advice, BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to add-advice");
			return "add-advice";
		}
		adviceService.addAdvice(advice);
		map.put("advice", new Advice());
		logger.info("go to add-advice");
		return "add-advice";
	}

	@RequestMapping(value = "/delete-advice-{adviceId}", method = RequestMethod.GET)
	public String deleteAdvice(@PathVariable("adviceId") Integer adviceId) {
		adviceService.removeAdvice(adviceId);
		logger.info("go to delete-advice-success for adviceId: " + adviceId);
		return "delete-advice-success";
	}

	@RequestMapping(value = "/update-advice-{adviceId}", method = RequestMethod.GET)
	public String updateAdvice(@PathVariable Integer adviceId, Map<String, Object> map) {
		map.put("advice", adviceService.getAdvice(adviceId));
		map.put("serviceList", serviceService.listService());
		logger.info("go to update-advice for adviceId: " + adviceId);
		return "update-advice";
	}

	@RequestMapping(value = "/update-advice-submit-{adviceId}", method = RequestMethod.POST)
	public String updateAdviceSubmit(@PathVariable Integer adviceId, @ModelAttribute @Valid Advice advice, BindingResult result) {
		advice.setAdviceId(adviceId);
		if (result.hasErrors()) {
			logger.info("result has errors go back to update-advice for adviceId: " + adviceId);
			return "update-advice";
		}
		advice.setAdviceId(adviceId);
		adviceService.updateAdvice(advice);
		logger.info("go to update-advice-success for adviceId: " + adviceId);
		return "update-advice-success";
	}
}
