package uk.co.trinitylogic.skyfall.controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import uk.co.trinitylogic.skyfall.service.MailService;

@Controller
public class AutoResponderController {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private MailService mailService;

	@RequestMapping(value = "/redirect-request-{delay}")
	public String redirectRequest(@PathVariable Integer delay, @RequestParam("email") String email, @RequestParam("forename") String forename, @RequestParam("filename") String filename,
			Map<String, Object> map, HttpServletRequest request) {
		String redirectUrl = "redirect:http://matthewwalton.co.uk/" + filename;
		logger.info("redirect request to: " + redirectUrl);
		sendEmail(email, forename, filename, delay);
		return redirectUrl;
	}

	// http://app.golfinsider.co.uk/Skyfall/redirect-request-generic-{delay}?email={email}&filename={filename}&forename={forename}&redirect={redirect}
	// http://app.golfinsider.co.uk/Skyfall/redirect-request-generic-5?email=sam1_jackson@hotmail.com&filename=golfinsider&forename=Samuel&redirect=http://app.golfinsider.co.uk/index.html

	@RequestMapping(value = "/redirect-request-generic-{delay}")
	public String redirectRequestGeneric(@PathVariable Integer delay, @RequestParam("email") String email, @RequestParam("forename") String forename, @RequestParam("filename") String filename,
			@RequestParam("redirect") String redirect, Map<String, Object> map, HttpServletRequest request) {
		String redirectUrl = "redirect:" + redirect;
		logger.info("redirect request to: " + redirectUrl);
		sendEmail(email, forename, filename, delay);
		return redirectUrl;
	}

	private void sendEmail(String email, String forename, String filename, int delay) {
		logger.info("send email after " + delay + "mins to email: " + email);
		Timer timer = new Timer();
		String bodyFile = "/home/skyfall/templates/" + filename + "-body.txt";
		String subjectFile = "/home/skyfall/templates/" + filename + "-subject.txt";
		String body = getAutoRepsonderEmailAsHtmlString(bodyFile);

		body = replacePlaceHolderForename(body, forename);
		body = replacePlaceHolderEmail(body, email);

		String subject = getAutoRepsonderEmailAsHtmlString(subjectFile);
		TimerTask task = new AutoResponderEmail(email, subject, body);
		timer.schedule(task, 1000 * 60 * delay);
	}

	class AutoResponderEmail extends TimerTask {

		private final String email;
		private final String subject;
		private final String body;

		public AutoResponderEmail(String email, String subject, String body) {
			this.email = email;
			this.subject = subject;
			this.body = body;
		}

		@Override
		public void run() {
			mailService.sendHtmlMail("matthew@matthewwalton.co.uk", email, subject, body);
		}

	}

	private String getAutoRepsonderEmailAsHtmlString(String filename) {
		BufferedReader br = null;
		StringBuffer sb = new StringBuffer();
		try {
			String currentLine;
			File file = new File(filename);
			br = new BufferedReader(new FileReader(file));
			while ((currentLine = br.readLine()) != null) {
				sb.append(currentLine);
			}
		} catch (IOException e) {
			logger.error(e);
		} finally {
			try {
				if (br != null) {
					br.close();
				}
			} catch (IOException ex) {
				logger.error(ex);
			}
		}
		return sb.toString();
	}

	public String replacePlaceHolderForename(String body, String forename) {
		return body.replaceAll("\\$Forename\\$", forename);
	}

	public String replacePlaceHolderEmail(String body, String email) {
		return body.replaceAll("\\$Email\\$", email);
	}

	public String replacePlaceHolderMobile(String body, String mobile) {
		return body.replaceAll("\\$Mobile\\$", mobile);
	}
}
