package uk.co.trinitylogic.skyfall.controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.apache.commons.lang.WordUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import uk.co.trinitylogic.skyfall.form.Enquirer;
import uk.co.trinitylogic.skyfall.form.Member;
import uk.co.trinitylogic.skyfall.form.Service;
import uk.co.trinitylogic.skyfall.service.EnquirerService;
import uk.co.trinitylogic.skyfall.service.MailService;
import uk.co.trinitylogic.skyfall.service.MemberService;
import uk.co.trinitylogic.skyfall.service.ServiceService;

@Controller
public class EnquirerController {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private EnquirerService enquirerService;

	@Autowired
	private MemberService memberService;

	@Autowired
	private ServiceService serviceService;

	@Autowired
	private MailService mailService;

	@RequestMapping("/list-enquirer")
	public String listEnquirer(Map<String, Object> map) {
		map.put("enquirerList", enquirerService.listEnquirer());
		logger.info("go to list-enquirer-success");
		return "list-enquirer-success";
	}

	@RequestMapping("/list-unsubscribes")
	public String listUnsubscribes(Map<String, Object> map) {
		map.put("unsubscribesList", enquirerService.listUnsubscribes());
		logger.info("go to list-unsubscribes-success");
		return "list-unsubscribes-success";
	}

	@RequestMapping("/unsubscribe")
	public String unsubscribe(Map<String, Object> map) {
		logger.info("go to unsubscribe");
		return "unsubscribe";
	}

	@RequestMapping("/optin-add-enquirer")
	public String addEnquirer(Map<String, Object> map) {
		map.put("enquirer", new Enquirer());
		logger.info("go to add-enquirer");
		return "add-enquirer";
	}

	@RequestMapping(value = "/optin-add-enquirer-submit", method = RequestMethod.POST)
	public String addEnquirerSubmit(@ModelAttribute @Valid Enquirer enquirer, BindingResult result, Map<String, Object> map, HttpServletRequest request) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to add-enquirer");
			return "add-enquirer";
		}
		return doAddEnquirer(enquirer, map, request);
	}

	@RequestMapping(value = "/optin-keithelliott")
	public String optinKeithelliott(@RequestParam("email") String email, @RequestParam("surname") String name, @RequestParam("RedirectURL") String redirectURL, Map<String, Object> map,
			HttpServletRequest request) {
		String nl = System.getProperty("line.separator");
		String message = "Name: " + name + nl + "Email: " + email + nl;
		mailService.sendMail("matthew@matthewwalton.co.uk", "matthew@matthewwalton.co.uk", "Enquirer From http://www.keithelliott.co.uk/", message);
		String redirect = "redirect:" + redirectURL;
		return redirect;
	}

	@RequestMapping(value = "/optin-enquirer")
	public String addEnquirerSubmit(@RequestParam("email") String email, @RequestParam("id") Integer id, Map<String, Object> map, HttpServletRequest request) {
		Enquirer enquirer = new Enquirer();
		enquirer.setEmail(email);
		enquirer.setServiceId(id);
		return doAddEnquirer(enquirer, map, request);
	}

	@RequestMapping(value = "/optout-enquirer", method = RequestMethod.POST)
	public String optOutEnquirer(@RequestParam("email") String email, Map<String, Object> map) {
		Member member = memberService.getMember(email);
		List<Enquirer> enquirer = enquirerService.getEnquirer(email);
		if (member != null) {
			member.setDoubleOptIn(false);
			Service bw = serviceService.getService(30);
			Service exbw = serviceService.getService(32);
			member.addService(exbw);
			member.removeService(bw);
			for (Enquirer enq : enquirer) {
				enq.setUnsubscribe(new Date());
				enquirerService.updateEnquirer(enq);
			}
			memberService.updateMember(member);
			return "redirect:http://matthewwalton.co.uk/betting-wages-opt-out-success/";
		} else {
			return "redirect:http://matthewwalton.co.uk/betting-wages-opt-out-failed/";
		}
	}

	public String doAddEnquirer(Enquirer enquirer, Map<String, Object> map, HttpServletRequest request) {
		boolean hasEnquired = enquirerService.hasEnquired(enquirer);
		String baseUrl = String.format("%s://%s:%d%s/optin-enquirer", request.getScheme(), request.getServerName(), request.getServerPort(), request.getContextPath());
		if (hasEnquired) {
			Enquirer enq = enquirerService.getEnquirer(enquirer.getEmail()).get(0);
			String url = baseUrl + "-" + enq.getEnquirerId();
			Member member = memberService.getMember(enquirer.getEmail());
			if (member != null) {
				map.put("member", member);
				if (!member.getDoubleOptIn()) {
					mailService.sendHtmlMail("matthew@matthewwalton.co.uk", enquirer.getEmail(), "Betting Wages: You’re still not signed up",
							getAutoRepsonderEmailHtmlAsString("autoResponderEmail4", url));
				} else {
					mailService.sendHtmlMail("matthew@matthewwalton.co.uk", enquirer.getEmail(), "Betting Wages: You’re already opted-in",
							getAutoRepsonderEmailHtmlAsString("autoResponderEmail5", null));
				}
			} else {
				mailService
						.sendHtmlMail("matthew@matthewwalton.co.uk", enquirer.getEmail(), "Betting Wages: You’re still not signed up", getAutoRepsonderEmailHtmlAsString("autoResponderEmail4", url));
			}
			return "redirect:http://matthewwalton.co.uk/betting-wages-opt-in/";
		}
		enquirerService.addEnquirer(enquirer);
		logger.info("send opt-in email");
		String optInUrl = baseUrl + "-" + enquirer.getEnquirerId();
		mailService.sendHtmlMail("matthew@matthewwalton.co.uk", enquirer.getEmail(), "Betting Wages: You MUST do this now", getAutoRepsonderEmailHtmlAsString("autoResponderEmail1", optInUrl));
		checkOptInComplete(enquirer, optInUrl);
		logger.info("go to betting-wages-opt-in");
		return "redirect:http://matthewwalton.co.uk/betting-wages-opt-in/";
	}

	@RequestMapping(value = "/optin-enquirer-{id}", method = RequestMethod.GET)
	public String optinEnquirer(@PathVariable Integer id, Map<String, Object> map, HttpServletRequest request) {
		logger.info("optin recieved for enquirer id: " + id);
		Enquirer enquirer = enquirerService.getEnquirer(id);
		Member member = memberService.getMember(enquirer.getEmail());
		if (member != null) {
			map.put("member", member);
			if (member.getDoubleOptIn()) {
				mailService.sendHtmlMail("matthew@matthewwalton.co.uk", enquirer.getEmail(), "Betting Wages: You’re already opted-in", getAutoRepsonderEmailHtmlAsString("autoResponderEmail5", null));
				Service service = serviceService.getService(enquirer.getServiceId());
				member.addService(service);
				memberService.updateMember(member);
				return "betting-wages-welcome";
			} else {
				member.setDoubleOptIn(true);
				Service service = serviceService.getService(enquirer.getServiceId());
				member.addService(service);
				memberService.updateMember(member);
			}
		} else {
			Member mem = new Member();
			mem.setEmail1(enquirer.getEmail());
			mem.setDoubleOptIn(true);
			Service service = serviceService.getService(enquirer.getServiceId());
			mem.addService(service);
			memberService.addMember(mem);
			map.put("member", mem);
		}
		logger.info("send opt-in email");
		member = memberService.getMember(enquirer.getEmail());
		String baseUrl = String.format("%s://%s:%d%s/betting-wages-preferences-", request.getScheme(), request.getServerName(), request.getServerPort(), request.getContextPath());
		String prefUrl = baseUrl + member.getMemberId();
		mailService.sendHtmlMail("matthew@matthewwalton.co.uk", enquirer.getEmail(), "A big, warm welcome to Betting Wages", getAutoRepsonderEmailHtmlAsString("autoResponderEmail2", prefUrl));
		return "betting-wages-welcome";
	}

	@RequestMapping(value = "/betting-wages-preferences-{id}", method = RequestMethod.GET)
	public String getEnquirer(@PathVariable Integer id, Map<String, Object> map) {
		logger.info("get details for for member id: " + id);
		Member member = memberService.getMember(id);
		if (member != null) {
			map.put("member", member);
		} else {
			logger.info("member not found return to: betting-wages-preferences-not-found");
			return "redirect:http://matthewwalton.co.uk/betting-wages-preferences-not-found";
		}
		logger.info("send to betting-wages-preferences");
		return "betting-wages-preferences";
	}

	@RequestMapping(value = "/betting-wages-preferences-submit-{id}", method = RequestMethod.POST)
	public String updateMamber(@PathVariable Integer id, @ModelAttribute @Valid Member member, BindingResult result) {
		Member mem = memberService.getMember(id);
		String mobile = member.getMobile();
		if (mobile != null) {
			mobile = mobile.replaceAll(" ", "").trim();
			if (mobile.startsWith("07")) {
				mobile = "447" + mobile.substring(2);
			}
		}
		String forename = member.getForename();
		if (forename != null) {
			forename = WordUtils.capitalizeFully(forename);
		}
		String surname = member.getSurname();
		if (surname != null) {
			surname = WordUtils.capitalizeFully(surname);
		}
		mem.setTitle(member.getTitle());
		mem.setForename(forename);
		mem.setSurname(surname);
		mem.setAddress1(member.getAddress1());
		mem.setAddress2(member.getAddress2());
		mem.setTown(member.getTown());
		mem.setCounty(member.getCounty());
		mem.setPostcode(member.getPostcode());
		mem.setCountry(member.getCountry());
		mem.setMobile(mobile);
		if (result.hasErrors()) {
			logger.info("result has errors go back to betting-wages-preferences for memberId: " + id);
			return "betting-wages-preferences";
		}
		memberService.updateMember(mem);
		logger.info("go to update-member-success for memberId: " + id);
		return "redirect:http://matthewwalton.co.uk/betting-wages-preferences-success";
	}

	private void checkOptInComplete(Enquirer enquirer, String optInUrl) {
		Timer timer = new Timer();
		TimerTask task = new OptInComplete(enquirer, optInUrl);
		timer.schedule(task, 1000 * 60 * 60);
	}

	class MatthewWalton extends TimerTask {

		private final String email;

		public MatthewWalton(String email) {
			this.email = email;
		}

		@Override
		public void run() {
			mailService.sendHtmlMail("matthew@matthewwalton.co.uk", email, "Introduction To Matthew Walton", "Introduction To Matthew Walton");
		}
	}

	class OptInComplete extends TimerTask {

		private final Enquirer enquirer;
		private final String optInUrl;

		public OptInComplete(Enquirer enquirer, String optInUrl) {
			this.enquirer = enquirer;
			this.optInUrl = optInUrl;
		}

		@Override
		public void run() {
			Member member = memberService.getMember(enquirer.getEmail());
			if (member != null) {
				if (!member.getDoubleOptIn()) {
					sendEmail();
				}
			} else {
				sendEmail();
			}
		}

		public void sendEmail() {
			mailService.sendHtmlMail("matthew@matthewwalton.co.uk", enquirer.getEmail(), "Betting Wages: You’re still not signed up",
					getAutoRepsonderEmailHtmlAsString("autoResponderEmail3", optInUrl));
		}
	}

	public String getAutoRepsonderEmailHtmlAsString(String filename, String optInUrl) {
		BufferedReader br = null;
		StringBuffer sb = new StringBuffer();
		try {
			String currentLine;
			File file = new File("/home/skyfall/templates/" + filename + ".txt");
			br = new BufferedReader(new FileReader(file));
			if (optInUrl != null) {
				while ((currentLine = br.readLine()) != null) {
					sb.append(currentLine.replaceAll("insertUrl", optInUrl));
				}
			}
		} catch (IOException e) {
			logger.error(e);
		} finally {
			try {
				if (br != null) {
					br.close();
				}
			} catch (IOException ex) {
				logger.error(ex);
			}
		}
		return sb.toString();
	}
}
