package uk.co.trinitylogic.skyfall.controller;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.multipart.MultipartFile;

import uk.co.trinitylogic.skyfall.form.Mail;
import uk.co.trinitylogic.skyfall.form.Member;
import uk.co.trinitylogic.skyfall.form.Rule;
import uk.co.trinitylogic.skyfall.form.Service;
import uk.co.trinitylogic.skyfall.form.Text;
import uk.co.trinitylogic.skyfall.service.MailService;
import uk.co.trinitylogic.skyfall.service.MemberService;
import uk.co.trinitylogic.skyfall.service.ServiceService;
import uk.co.trinitylogic.skyfall.service.TxtService;

@Controller
@SessionAttributes({ "rule", "mail", "text" })
public class MailController {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private MemberService memberService;

	@Autowired
	private ServiceService serviceService;

	@Autowired
	private MailService mailService;

	@Autowired
	private TxtService txtService;

	@RequestMapping("/mail-member-{memberId}")
	public String mailMember(@PathVariable Integer memberId, Map<String, Object> map) {
		Member member = memberService.getMember(memberId);
		Mail mail = new Mail();
		mail.setMember(member);
		map.put("mail", mail);
		logger.info("go to mail-member");
		return "mail-member";
	}

	@RequestMapping("/mail-member-submit")
	public String mailMemberSubmit(@ModelAttribute("mail") @Valid Mail mail, BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to mail-member");
			return "mail-member";
		}
		mailService.sendMail("matthew@matthewwalton.co.uk", mail.getMember().getEmail1(), "matthew@matthewwalton.co.uk", mail.getSubject(), mail.getBody());
		logger.info("go to mail-member-success");
		return "mail-member-success";
	}

	@RequestMapping("/mail-members")
	public String mailMembers(Map<String, Object> map) {
		map.put("serviceList", serviceService.listService());
		map.put("mail", new Mail());
		logger.info("go to mail-members");
		return "mail-members";
	}

	@RequestMapping("/mail-members-submit")
	public String mailMembersSubmit(@ModelAttribute("mail") Mail mail, BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to mail-members");
			return "mail-members";
		}
		List<Member> members = new ArrayList<Member>();
		MultipartFile csvFile = mail.getCsvFile();
		BufferedReader br = null;
		String line = "";
		try {
			br = new BufferedReader(new InputStreamReader(csvFile.getInputStream()));
			while ((line = br.readLine()) != null) {
				String[] member = line.split(",");
				logger.info("Forename|Email [" + member[0] + "|" + member[2] + "]");
				Member mem = new Member();
				mem.setForename(member[0]);
				mem.setSurname(member[1]);
				mem.setEmail1(member[2]);
				members.add(mem);
			}
		} catch (Exception e) {
			logger.error(e);
		} finally {
			if (br != null) {
				try {
					br.close();
				} catch (IOException e) {
					logger.error(e);
				}
			}
		}
		List<Service> services = mail.getServices();
		if (services != null && services.size() != 0) {
			List<Member> serviceMembers;
			if (mail.isDoubleOptIn()) {
				serviceMembers = memberService.listDistinctMembersForMailServiceDoubleOptIn(mail.getServices());
			} else {
				serviceMembers = memberService.listDistinctMembersForMailServiceOptIn(mail.getServices());
			}
			for (Member m : serviceMembers) {
				members.add(m);
			}
		}
		boolean html = false;
		String body = mail.getBody().trim();
		if (body.startsWith("<")) {
			html = true;
		}
		for (Member m : members) {
			body = mail.getBody().trim();
			if (m.getForename() != null) {
				body = body.replaceAll("\\$Forename\\$", m.getForename());
			} else {
				body = body.replaceAll("\\$Forename\\$", "member");
			}
			if (m.getEmail1() != null) {
				body = body.replaceAll("\\$Email\\$", m.getEmail1());
			}
			if (m.getMobile() != null) {
				body = body.replaceAll("\\$Mobile\\$", m.getMobile());
			}
			if (html) {
				mailService.sendHtmlMail("matthew@matthewwalton.co.uk", m.getEmail1(), mail.getSubject(), body);
			} else {
				mailService.sendMail("matthew@matthewwalton.co.uk", m.getEmail1(), mail.getSubject(), body);
			}
		}
		map.put("mailMembers", members);
		logger.info("go to mail-members-success");
		return "mail-members-success";
	}

	@RequestMapping("/rule-members")
	public String ruleMembers(Model model) {
		List<Service> service1 = serviceService.listService();
		List<Service> service2 = new ArrayList<Service>(service1);
		model.addAttribute("service1", service1);
		model.addAttribute("service2", service2);
		model.addAttribute("rule", new Rule());
		model.addAttribute("mail", new Mail());
		model.addAttribute("text", new Text());
		logger.info("go to rule-members");
		return "rule-members";
	}

	@RequestMapping("/rule-members-submit")
	public String mailMembersSubmit(@ModelAttribute("rule") @Valid Rule rule, BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to rule-members");
			return "rule-members";
		}
		List<Member> members1 = memberService.listDistinctMembersForServiceList(rule.getServices1());
		List<Member> members2 = memberService.listDistinctMembersForServiceList(rule.getServices2());
		List<Member> match = new ArrayList<Member>();
		List<Member> notmatch = new ArrayList<Member>();
		boolean b = false;
		for (Member m1 : members1) {
			for (Member m2 : members2) {
				if (m1.equals(m2)) {
					match.add(m2);
					b = true;
				}
			}
			if (b != true) {
				notmatch.add(m1);
			} else {
				b = false;
			}
		}
		rule.setMatch(match);
		rule.setNotmatch(notmatch);
		logger.info("go to rule-members-success");
		return "rule-members-success";
	}

	@RequestMapping("/mail-list-members-{typeID}")
	public String mailListMembers(@ModelAttribute("mail") Mail mail, @ModelAttribute("rule") @Valid Rule rule, @PathVariable String typeID, Map<String, Object> map) {
		if ("M".equalsIgnoreCase(typeID)) {
			mail.setList(rule.getMatch());
		}
		if ("N".equalsIgnoreCase(typeID)) {
			mail.setList(rule.getNotmatch());
		}
		logger.info("go to mail-list-members");
		return "mail-list-members";
	}

	@RequestMapping("/mail-list-members-submit")
	public String mailListMembersSubmit(@ModelAttribute("mail") @Valid Mail mail, BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to mail-list-members");
			return "mail-list-members";
		}
		List<Member> members = mail.getList();
		for (Member m : members) {
			if (m.getEmail1() != null && !"".equalsIgnoreCase(m.getEmail1())) {
				mailService.sendMail("matthew@matthewwalton.co.uk", m.getEmail1(), mail.getSubject(), mail.getBody());
			}
		}
		map.put("mailMembers", members);
		logger.info("go to mail-list-members-success");
		return "mail-list-members-success";
	}

	@RequestMapping("/text-member-{memberId}")
	public String textMember(@PathVariable Integer memberId, Map<String, Object> map) {
		Member member = memberService.getMember(memberId);
		Text text = new Text();
		text.setMember(member);
		map.put("text", text);
		logger.info("go to text-member");
		return "text-member";
	}

	@RequestMapping("/text-member-submit")
	public String textMembersubmit(@ModelAttribute("text") @Valid Text text, BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to text-members");
			return "text-members";
		}

		String body = text.getText();
		Member member = text.getMember();
		if (member.getForename() != null) {
			body = body.replaceAll("\\$Forename\\$", member.getForename());
		} else {
			body = body.replaceAll("\\$Forename\\$", "member");
		}

		txtService.sendTxt(member.getMobile(), body);
		logger.info("go to text-member-success");
		return "text-member-success";
	}

	@RequestMapping("/text-members")
	public String textMembers(Map<String, Object> map) {
		map.put("serviceList", serviceService.listService());
		map.put("text", new Text());
		logger.info("go to text-members");
		return "text-members";
	}

	@RequestMapping("/text-members-submit")
	public String textMembersSubmit(@ModelAttribute("text") Text text, BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to text-members");
			return "text-members";
		}
		List<Member> members = new ArrayList<Member>();
		MultipartFile csvFile = text.getCsvFile();
		BufferedReader br = null;
		String line = "";
		try {
			br = new BufferedReader(new InputStreamReader(csvFile.getInputStream()));
			while ((line = br.readLine()) != null) {
				String[] member = line.split(",");
				logger.info("Forename|Mobile [" + member[0] + "|" + member[1] + "]");
				Member mem = new Member();
				mem.setForename(member[0]);
				mem.setMobile(member[1]);
				members.add(mem);
			}
		} catch (Exception e) {
			logger.error(e);
		} finally {
			if (br != null) {
				try {
					br.close();
				} catch (IOException e) {
					logger.error(e);
				}
			}
		}

		List<Service> services = text.getServices();
		if (services != null && services.size() != 0) {
			List<Member> serviceMembers = memberService.listDistinctMembersForTextServiceOptIn(services);
			for (Member m : serviceMembers) {
				members.add(m);
			}
		}

		for (Member member : members) {

			String body = text.getText();
			if (member.getForename() != null) {
				body = body.replaceAll("\\$Forename\\$", member.getForename());
			} else {
				body = body.replaceAll("\\$Forename\\$", "member");
			}

			txtService.sendTxt(member.getMobile(), body);
		}
		map.put("textMembers", members);
		logger.info("go to text-members-success");
		return "text-members-success";
	}

	@RequestMapping("/text-list-members-{typeID}")
	public String textListMembers(@ModelAttribute("text") Text text, @ModelAttribute("rule") @Valid Rule rule, @PathVariable String typeID, Map<String, Object> map) {
		if ("M".equalsIgnoreCase(typeID)) {
			text.setList(rule.getMatch());
		}
		if ("N".equalsIgnoreCase(typeID)) {
			text.setList(rule.getNotmatch());
		}
		logger.info("go to text-list-members");
		return "text-list-members";
	}

	@RequestMapping("/text-list-members-submit")
	public String textListMembersSubmit(@ModelAttribute("text") @Valid Text text, BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to text-list-members");
			return "text-list-members";
		}
		List<Member> members = text.getList();
		for (Member member : members) {
			if (member.getMobile() != null && !"".equalsIgnoreCase(member.getMobile())) {

				String body = text.getText();
				if (member.getForename() != null) {
					body = body.replaceAll("\\$Forename\\$", member.getForename());
				} else {
					body = body.replaceAll("\\$Forename\\$", "member");
				}

				txtService.sendTxt(member.getMobile(), body);
			}
		}
		map.put("textMembers", members);
		logger.info("go to text-list-members-success");
		return "text-list-members-success";
	}

}
