package uk.co.trinitylogic.skyfall.controller;

import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import uk.co.trinitylogic.skyfall.form.Enquirer;
import uk.co.trinitylogic.skyfall.form.Member;
import uk.co.trinitylogic.skyfall.service.EnquirerService;
import uk.co.trinitylogic.skyfall.service.MemberService;
import uk.co.trinitylogic.skyfall.service.ServiceService;

@Controller
public class MemberController {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private MemberService memberService;

	@Autowired
	private ServiceService serviceService;

	@Autowired
	private EnquirerService enquirerService;

	@RequestMapping("/list-member")
	public String listMember(Map<String, Object> map) {
		map.put("memberList", memberService.listMember());
		logger.info("go to list-member-success");
		return "list-member-success";
	}

	@RequestMapping("/search-member-submit")
	public String searchMember(Map<String, Object> map,
			@RequestParam("searchMember") String searchMember) {
		logger.info("go to search-member");
		map.put("searchMemberList",
				memberService.searchForMembers((searchMember)));
		return "search-member";
	}

	@RequestMapping("/add-member")
	public String addMember(Map<String, Object> map) {
		map.put("serviceList", serviceService.listService());
		map.put("member", new Member());
		logger.info("go to add-member");
		return "add-member";
	}

	@RequestMapping(value = "/add-member-submit", method = RequestMethod.POST)
	public String addMemberSubmit(
			@ModelAttribute("member") @Valid Member member, BindingResult result) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to add-member");
			return "add-member";
		}
		memberService.addMember(member);
		logger.info("go to add-member-success");
		return "add-member-success";
	}

	@RequestMapping(value = "/delete-member-{memberId}", method = RequestMethod.GET)
	public String deleteMember(@PathVariable Integer memberId,
			Map<String, Object> map) {
		map.put("member", memberService.getMember(memberId));
		memberService.removeMember(memberId);
		logger.info("go to delete-member-success for memberId: " + memberId);
		return "delete-member-success";
	}

	@RequestMapping(value = "/update-member-{memberId}", method = RequestMethod.GET)
	public String updateMember(@PathVariable Integer memberId,
			Map<String, Object> map) {
		Member member = memberService.getMember(memberId);
		List<Enquirer> enquirer = enquirerService.getEnquirer(member
				.getEmail1());
		map.put("serviceList", serviceService.listService());
		map.put("member", member);
		map.put("enquirer", enquirer);
		logger.info("go to update-member for memberId: " + memberId);
		return "update-member";
	}

	@RequestMapping(value = "/update-member-submit-{memberId}", method = RequestMethod.POST)
	public String updateMemberSubmit(@PathVariable Integer memberId,
			@ModelAttribute @Valid Member member, BindingResult result) {
		member.setMemberId(memberId);
		if (result.hasErrors()) {
			logger.info("result has errors go back to update-member for memberId: "
					+ memberId);
			return "update-member";
		}
		memberService.updateMember(member);
		logger.info("go to update-member-success for memberId: " + memberId);
		return "update-member-success";
	}
}
