package uk.co.trinitylogic.skyfall.controller;

import java.util.Map;

import javax.validation.Valid;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import uk.co.trinitylogic.skyfall.form.Transaction;
import uk.co.trinitylogic.skyfall.service.TransactionService;
import urn.ebay.api.PayPalAPI.CreateRecurringPaymentsProfileReq;
import urn.ebay.api.PayPalAPI.CreateRecurringPaymentsProfileRequestType;
import urn.ebay.api.PayPalAPI.CreateRecurringPaymentsProfileResponseType;
import urn.ebay.api.PayPalAPI.DoDirectPaymentReq;
import urn.ebay.api.PayPalAPI.DoDirectPaymentRequestType;
import urn.ebay.api.PayPalAPI.DoDirectPaymentResponseType;
import urn.ebay.api.PayPalAPI.PayPalAPIInterfaceServiceService;
import urn.ebay.apis.CoreComponentTypes.BasicAmountType;
import urn.ebay.apis.eBLBaseComponents.BillingPeriodDetailsType;
import urn.ebay.apis.eBLBaseComponents.BillingPeriodType;
import urn.ebay.apis.eBLBaseComponents.CreateRecurringPaymentsProfileRequestDetailsType;
import urn.ebay.apis.eBLBaseComponents.CreditCardDetailsType;
import urn.ebay.apis.eBLBaseComponents.CreditCardTypeType;
import urn.ebay.apis.eBLBaseComponents.CurrencyCodeType;
import urn.ebay.apis.eBLBaseComponents.DoDirectPaymentRequestDetailsType;
import urn.ebay.apis.eBLBaseComponents.PayerInfoType;
import urn.ebay.apis.eBLBaseComponents.PaymentActionCodeType;
import urn.ebay.apis.eBLBaseComponents.PaymentDetailsType;
import urn.ebay.apis.eBLBaseComponents.PersonNameType;
import urn.ebay.apis.eBLBaseComponents.RecurringPaymentsProfileDetailsType;
import urn.ebay.apis.eBLBaseComponents.ScheduleDetailsType;
import util.Configuration;

@Controller
public class PayPalController {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private TransactionService transactionService;

	// http://localhost:8080/Skyfall/get-secure-checkout?price=1.00&vat=0.17&total=1.17&service=ProBacker&billingFrequency=12&billingPeriod=Month&totalBillingCycles=0
	@RequestMapping("/get-secure-checkout")
	public String getSecureCheckout(@RequestParam("price") String price,
			@RequestParam("vat") String vat,
			@RequestParam("total") String total,
			@RequestParam("service") String service,
			@RequestParam("billingFrequency") String billingFrequency,
			@RequestParam("billingPeriod") String billingPeriod,
			@RequestParam("totalBillingCycles") String totalBillingCycles,
			Map<String, Object> map) {
		Transaction secureCheckout = new Transaction();
		secureCheckout.setPrice(price);
		secureCheckout.setVat(vat);
		secureCheckout.setTotal(total);
		secureCheckout.setService(service);
		secureCheckout.setBillingFrequency(billingFrequency);
		secureCheckout.setBillingPeriod(billingPeriod);
		secureCheckout.setTotalBillingCycles(totalBillingCycles);

		map.put("checkout", secureCheckout);
		logger.info("go to secure-checkout");
		return "secure-checkout";
	}

	// http://localhost:8080/Skyfall/get-secure-trial?price=1.00&vat=0.17&total=1.17&service=ProBacker&billingFrequency=12&billingPeriod=Month&totalBillingCycles=0&trialBillingAmount=0&trialBillingFrequency=12&trialBillingPeriod=Month&trialBillingCycles=1&trialTaxAmount=0&trialTotalAmount=0
	@RequestMapping("/get-secure-trial")
	public String getSecureCheckoutTrial(
			@RequestParam("price") String price,
			@RequestParam("vat") String vat,
			@RequestParam("total") String total,
			@RequestParam("service") String service,
			@RequestParam("billingFrequency") String billingFrequency,
			@RequestParam("billingPeriod") String billingPeriod,
			@RequestParam("totalBillingCycles") String totalBillingCycles,
			@RequestParam("trialBillingAmount") String trialBillingAmount,
			@RequestParam("trialBillingFrequency") String trialBillingFrequency,
			@RequestParam("trialBillingPeriod") String trialBillingPeriod,
			@RequestParam("trialBillingCycles") String trialBillingCycles,
			@RequestParam("trialTaxAmount") String trialTaxAmount,
			@RequestParam("trialTotalAmount") String trialTotalAmount,

			Map<String, Object> map) {
		try {
			Transaction secureCheckout = new Transaction();
			secureCheckout.setPrice(price);
			secureCheckout.setVat(vat);
			secureCheckout.setTotal(total);
			secureCheckout.setService(service);
			secureCheckout.setBillingFrequency(billingFrequency);
			secureCheckout.setBillingPeriod(billingPeriod);
			secureCheckout.setTotalBillingCycles(totalBillingCycles);

			secureCheckout.setTrialBillingAmount(trialBillingAmount);
			secureCheckout.setTrialBillingFrequency(trialBillingFrequency);
			secureCheckout.setTrialBillingPeriod(trialBillingPeriod);
			secureCheckout.setTrialBillingCycles(trialBillingCycles);
			secureCheckout.setTrialTaxAmount(trialTaxAmount);
			secureCheckout.setTrialTotalAmount(trialTotalAmount);

			map.put("checkout", secureCheckout);
			logger.info("go to secure-trial");
		} catch (Exception ex) {
			logger.error(ex);
		}
		return "secure-trial";
	}

	// http://localhost:8080/Skyfall/get-secure-single-payment?price=1.00&vat=0.17&total=1.17&service=ProBacker
	@RequestMapping("/get-secure-single-payment")
	public String getSecureSinglePayment(@RequestParam("price") String price,
			@RequestParam("vat") String vat,
			@RequestParam("total") String total,
			@RequestParam("service") String service, Map<String, Object> map) {
		Transaction secureCheckout = new Transaction();
		secureCheckout.setPrice(price);
		secureCheckout.setVat(vat);
		secureCheckout.setTotal(total);
		secureCheckout.setService(service);
		map.put("checkout", secureCheckout);
		logger.info("go to get-secure-single-payment");
		return "secure-single-payment";
	}

	@RequestMapping("/list-transaction")
	public String listTransaction(Map<String, Object> map) {
		map.put("transactionList", transactionService.listTransaction());
		logger.info("go to list-transaction-success");
		return "list-transaction-success";
	}

	@RequestMapping("/get-secure-checkout-success")
	public String getSecureCheckoutSuccess(Map<String, Object> map) {
		logger.info("go to secure-checkout-success");
		return "secure-checkout-success";
	}

	@RequestMapping("/get-secure-checkout-fail")
	public String getSecureCheckoutSuccessFail(Map<String, Object> map) {
		logger.info("go to secure-checkout-fail");
		return "secure-checkout-fail";
	}

	@RequestMapping("/do-secure-checkout")
	public String doSecureCheckout(
			@ModelAttribute("checkout") @Valid Transaction checkout,
			BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to secure-checkout");
			return "secure-checkout";
		}

		CurrencyCodeType currency = CurrencyCodeType.fromValue("GBP");
		logger.info("Currency: " + currency);
		Map<String, String> configurationMap = Configuration.getAcctAndConfig();
		PayPalAPIInterfaceServiceService service = new PayPalAPIInterfaceServiceService(
				configurationMap);

		CreateRecurringPaymentsProfileReq req = new CreateRecurringPaymentsProfileReq();
		CreateRecurringPaymentsProfileRequestType reqType = new CreateRecurringPaymentsProfileRequestType();
		/*
		 * (Required) The date when billing for this profile begins. Note: The
		 * profile may take up to 24 hours for activation. Character length and
		 * limitations: Must be a valid date, in UTC/GMT format
		 */
		String billingStartDate = checkout.getBillingStartDate();
		logger.info("BillingStartDate: " + billingStartDate);
		RecurringPaymentsProfileDetailsType profileDetails = new RecurringPaymentsProfileDetailsType(
				billingStartDate + "T00:00:00:000Z");

		ScheduleDetailsType scheduleDetails = new ScheduleDetailsType();
		/*
		 * (Required) Description of the recurring payment. Note: You must
		 * ensure that this field matches the corresponding billing agreement
		 * description included in the SetExpressCheckout request. Character
		 * length and limitations: 127 single-byte alphanumeric characters
		 */
		String profileDescription = checkout.getService();
		logger.info("ProfileDescription: " + profileDescription);
		scheduleDetails.setDescription(profileDescription);

		/*
		 * (Required) Number of billing periods that make up one billing cycle.
		 * The combination of billing frequency and billing period must be less
		 * than or equal to one year. For example, if the billing cycle is
		 * Month, the maximum value for billing frequency is 12. Similarly, if
		 * the billing cycle is Week, the maximum value for billing frequency is
		 * 52. Note: If the billing period is SemiMonth, the billing frequency
		 * must be 1.
		 */
		String billingFrequency = checkout.getBillingFrequency();
		logger.info("BillingFrequency: " + billingFrequency);

		int frequency = Integer.parseInt(billingFrequency);
		/*
		 * (Required) Billing amount for each billing cycle during this payment
		 * period. This amount does not include shipping and tax amounts. Note:
		 * All amounts in the CreateRecurringPaymentsProfile request must have
		 * the same currency. Character length and limitations: Value is a
		 * positive number which cannot exceed $10,000 USD in any currency. It
		 * includes no currency symbol. It must have 2 decimal places, the
		 * decimal separator must be a period (.), and the optional thousands
		 * separator must be a comma (,).
		 */
		String billingAmount = checkout.getPrice();
		logger.info("BillingAmount: " + billingAmount);

		BasicAmountType paymentAmount = new BasicAmountType(currency,
				billingAmount);
		/*
		 * (Required) Unit for billing during this subscription period. It is
		 * one of the following values: [Day, Week, SemiMonth, Month, Year] For
		 * SemiMonth, billing is done on the 1st and 15th of each month. Note:
		 * The combination of BillingPeriod and BillingFrequency cannot exceed
		 * one year.
		 */
		String billingPeriod = checkout.getBillingPeriod();
		logger.info("BillingPeriod: " + billingPeriod);

		BillingPeriodType period = BillingPeriodType.fromValue(billingPeriod);
		logger.info("BillingPeriodType: " + period.toString());
		/*
		 * (Optional) Number of billing cycles for payment period. For the
		 * regular payment period, if no value is specified or the value is 0,
		 * the regular payment period continues until the profile is cancelled
		 * or deactivated. For the regular payment period, if the value is
		 * greater than 0, the regular payment period will expire after the
		 * trial period is finished and continue at the billing frequency for
		 * TotalBillingCycles cycles.
		 */
		String totalBillingCycles = checkout.getTotalBillingCycles();
		logger.info("TotalBillingCycles: " + totalBillingCycles);
		BillingPeriodDetailsType paymentPeriod = new BillingPeriodDetailsType(
				period, frequency, paymentAmount);
		try {
			int numCycles = Integer.parseInt(totalBillingCycles);
			paymentPeriod.setTotalBillingCycles(numCycles);
		} catch (Exception e) {
			logger.error("Problem getting the Total Billing Cycles", e);
		}

		/*
		 * (Optional) Tax amount for each billing cycle during this payment
		 * period. Note: All amounts in the request must have the same currency.
		 * Character length and limitations: Value is a positive number which
		 * cannot exceed $10,000 USD in any currency. It includes no currency
		 * symbol. It must have 2 decimal places, the decimal separator must be
		 * a period (.), and the optional thousands separator must be a comma
		 * (,).
		 */
		String taxAmount = checkout.getVat();
		logger.info("TaxAmount: " + taxAmount);

		paymentPeriod.setTaxAmount(new BasicAmountType(currency, taxAmount));

		scheduleDetails.setPaymentPeriod(paymentPeriod);

		CreateRecurringPaymentsProfileRequestDetailsType reqDetails = new CreateRecurringPaymentsProfileRequestDetailsType(
				profileDetails, scheduleDetails);
		/*
		 * credit card number is required for CreateRecurringPaymentsProfile.
		 * Each CreateRecurringPaymentsProfile request creates a single
		 * recurring payments profile. Note:
		 */
		CreditCardDetailsType cc = new CreditCardDetailsType();

		/*
		 * (Required) Credit card number. Character length and limitations:
		 * Numeric characters only with no spaces or punctuation. The string
		 * must conform with modulo and length required by each credit card
		 * type.
		 */
		String creditCardNumber = checkout.getCardNumber();
		logger.info("CreditCardNumber: " + creditCardNumber);

		cc.setCreditCardNumber(creditCardNumber);

		/*
		 * Card Verification Value, version 2. Your Merchant Account settings
		 * determine whether this field is required. To comply with credit card
		 * processing regulations, you must not store this value after a
		 * transaction has been completed. Character length and limitations: For
		 * Visa, MasterCard, and Discover, the value is exactly 3 digits. For
		 * American Express, the value is exactly 4 digits.
		 */
		String cvv = checkout.getSecurityNumber();
		logger.info("CVV: " + cvv);

		cc.setCVV2(cvv);

		String issue = checkout.getIssue();
		if (issue != null && !issue.equals("")) {
			cc.setIssueNumber(issue);
		}

		// Start Month
		String startMonth = checkout.getStartMonth();
		logger.info("StartMonth: " + startMonth);

		cc.setStartMonth(Integer.parseInt(startMonth));

		// Start Year
		String startYear = checkout.getStartYear();
		logger.info("StartYear: " + startYear);

		cc.setStartYear(Integer.parseInt(startYear));

		// Expiry Month
		String expMonth = checkout.getEndMonth();
		logger.info("ExpMonth: " + expMonth);

		cc.setExpMonth(Integer.parseInt(expMonth));

		// Expiry Year
		String expYear = checkout.getEndYear();
		logger.info("ExpYear: " + expYear);

		cc.setExpYear(Integer.parseInt(expYear));
		PayerInfoType payerInfo = new PayerInfoType();

		/*
		 * (Required) Email address of buyer. Character length and limitations:
		 * 127 single-byte characters
		 */
		String buyerEmailId = checkout.getEmail();
		logger.info("BuyerEmailId: " + buyerEmailId);

		payerInfo.setPayer(buyerEmailId);

		String forname = checkout.getForename();
		String surname = checkout.getSurname();
		String mobile = checkout.getMobile();

		payerInfo.setContactPhone(mobile);

		PersonNameType payerName = new PersonNameType();
		payerName.setFirstName(forname);
		payerName.setLastName(surname);
		payerInfo.setPayerName(payerName);
		cc.setCardOwner(payerInfo);
		/*
		 * (Optional) Type of credit card. For UK, only Maestro, MasterCard,
		 * Discover, and Visa are allowable. For Canada, only MasterCard and
		 * Visa are allowable and Interac debit cards are not supported. It is
		 * one of the following values: [ Visa, MasterCard, Discover, Amex,
		 * Maestro: See note.] Note: If the credit card type is Maestro, you
		 * must set CURRENCYCODE to GBP. In addition, you must specify either
		 * STARTDATE or ISSUENUMBER.
		 */
		String creditCardType = checkout.getCreditCardType();
		logger.info("CreditCardType: " + creditCardType);

		CreditCardTypeType type = CreditCardTypeType.fromValue(creditCardType);
		switch (type) {
		case AMEX:
			cc.setCreditCardType(CreditCardTypeType.MAESTRO);
			break;
		case VISA:
			cc.setCreditCardType(CreditCardTypeType.VISA);
			break;
		case MASTERCARD:
			cc.setCreditCardType(CreditCardTypeType.MASTERCARD);
			break;
		default:
			break;
		}

		reqDetails.setCreditCard(cc);

		reqType.setCreateRecurringPaymentsProfileRequestDetails(reqDetails);
		req.setCreateRecurringPaymentsProfileRequest(reqType);
		CreateRecurringPaymentsProfileResponseType resp = null;
		try {
			resp = service.createRecurringPaymentsProfile(req);
		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		if (resp != null) {
			logger.info("lastReq: " + service.getLastRequest());
			logger.info("lastResp: " + service.getLastResponse());

			checkout.setRequest(service.getLastRequest());
			checkout.setResponse(service.getLastResponse());

			if (resp.getAck().toString().equalsIgnoreCase("SUCCESS")) {
				logger.info("Ack: " + resp.getAck());
				map.put("Ack", resp.getAck());
				/*
				 * (Required) Recurring payments profile ID returned in the
				 * CreateRecurringPaymentsProfile response. Note: The profile
				 * must have a status of either Active or Suspended. Character
				 * length and limitations: 14 single-byte alphanumeric
				 * characters. 19 character profile IDs are supported for
				 * compatibility with previous versions of the PayPal API.
				 */
				map.put("profileID", resp
						.getCreateRecurringPaymentsProfileResponseDetails()
						.getProfileID());
				logger.info("Profile ID: "
						+ resp.getCreateRecurringPaymentsProfileResponseDetails()
								.getProfileID());

				map.put("transactionID", resp
						.getCreateRecurringPaymentsProfileResponseDetails()
						.getTransactionID());
				logger.info("Transaction ID: "
						+ resp.getCreateRecurringPaymentsProfileResponseDetails()
								.getTransactionID());
				/*
				 * Status of the recurring payment profile. ActiveProfile – The
				 * recurring payment profile has been successfully created and
				 * activated for scheduled payments according the billing
				 * instructions from the recurring payments profile.
				 * PendingProfile – The system is in the process of creating the
				 * recurring payment profile. Please check your IPN messages for
				 * an update.
				 */
				map.put("Profile Status", resp
						.getCreateRecurringPaymentsProfileResponseDetails()
						.getProfileStatus());
				logger.info("Profile Status: "
						+ resp.getCreateRecurringPaymentsProfileResponseDetails()
								.getProfileStatus());
				transactionService.addTransaction(checkout);
				return "secure-checkout-success";
			} else {
				logger.info("error: " + resp.getErrors());
				map.put("error", resp.getErrors());
				transactionService.addTransaction(checkout);
				return "secure-checkout-fail";
			}
		}

		logger.info("go to do-secure-checkout-success");
		return "secure-checkout-success";
	}

	@RequestMapping("/do-secure-single-payment")
	public String doSecureSinglePayment(
			@ModelAttribute("checkout") @Valid Transaction checkout,
			BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to secure-checkout");
			return "secure-checkout";
		}

		CurrencyCodeType currency = CurrencyCodeType.fromValue("GBP");
		logger.info("Currency: " + currency);
		Map<String, String> configurationMap = Configuration.getAcctAndConfig();
		PayPalAPIInterfaceServiceService service = new PayPalAPIInterfaceServiceService(
				configurationMap);

		DoDirectPaymentReq doPaymentReq = new DoDirectPaymentReq();

		DoDirectPaymentRequestType pprequest = new DoDirectPaymentRequestType();
		DoDirectPaymentRequestDetailsType details = new DoDirectPaymentRequestDetailsType();
		PaymentDetailsType paymentDetails = new PaymentDetailsType();

		String profileDescription = checkout.getService();
		logger.info("ProfileDescription: " + profileDescription);

		String billingAmount = checkout.getTotal();
		logger.info("BillingAmount: " + billingAmount);

		BasicAmountType paymentAmount = new BasicAmountType(currency,
				billingAmount);
		paymentDetails.setOrderTotal(paymentAmount);
		String taxAmount = checkout.getVat();
		logger.info("TaxAmount: " + taxAmount);

		/*
		 * credit card number is required for CreateRecurringPaymentsProfile.
		 * Each CreateRecurringPaymentsProfile request creates a single
		 * recurring payments profile. Note:
		 */
		CreditCardDetailsType cc = new CreditCardDetailsType();

		/*
		 * (Required) Credit card number. Character length and limitations:
		 * Numeric characters only with no spaces or punctuation. The string
		 * must conform with modulo and length required by each credit card
		 * type.
		 */
		String creditCardNumber = checkout.getCardNumber();
		logger.info("CreditCardNumber: " + creditCardNumber);

		cc.setCreditCardNumber(creditCardNumber);

		/*
		 * Card Verification Value, version 2. Your Merchant Account settings
		 * determine whether this field is required. To comply with credit card
		 * processing regulations, you must not store this value after a
		 * transaction has been completed. Character length and limitations: For
		 * Visa, MasterCard, and Discover, the value is exactly 3 digits. For
		 * American Express, the value is exactly 4 digits.
		 */
		String cvv = checkout.getSecurityNumber();
		logger.info("CVV: " + cvv);

		cc.setCVV2(cvv);

		String issue = checkout.getIssue();
		if (issue != null && !issue.equals("")) {
			cc.setIssueNumber(issue);
		}

		// Start Month
		String startMonth = checkout.getStartMonth();
		logger.info("StartMonth: " + startMonth);

		cc.setStartMonth(Integer.parseInt(startMonth));

		// Start Year
		String startYear = checkout.getStartYear();
		logger.info("StartYear: " + startYear);

		cc.setStartYear(Integer.parseInt(startYear));

		// Expiry Month
		String expMonth = checkout.getEndMonth();
		logger.info("ExpMonth: " + expMonth);

		cc.setExpMonth(Integer.parseInt(expMonth));

		// Expiry Year
		String expYear = checkout.getEndYear();
		logger.info("ExpYear: " + expYear);

		cc.setExpYear(Integer.parseInt(expYear));
		PayerInfoType payerInfo = new PayerInfoType();

		/*
		 * (Required) Email address of buyer. Character length and limitations:
		 * 127 single-byte characters
		 */
		String buyerEmailId = checkout.getEmail();
		logger.info("BuyerEmailId: " + buyerEmailId);

		payerInfo.setPayer(buyerEmailId);

		String forname = checkout.getForename();
		String surname = checkout.getSurname();
		String mobile = checkout.getMobile();

		payerInfo.setContactPhone(mobile);

		PersonNameType payerName = new PersonNameType();
		payerName.setFirstName(forname);
		payerName.setLastName(surname);
		payerInfo.setPayerName(payerName);
		cc.setCardOwner(payerInfo);
		/*
		 * (Optional) Type of credit card. For UK, only Maestro, MasterCard,
		 * Discover, and Visa are allowable. For Canada, only MasterCard and
		 * Visa are allowable and Interac debit cards are not supported. It is
		 * one of the following values: [ Visa, MasterCard, Discover, Amex,
		 * Maestro: See note.] Note: If the credit card type is Maestro, you
		 * must set CURRENCYCODE to GBP. In addition, you must specify either
		 * STARTDATE or ISSUENUMBER.
		 */
		String creditCardType = checkout.getCreditCardType();
		logger.info("CreditCardType: " + creditCardType);

		CreditCardTypeType type = CreditCardTypeType.fromValue(creditCardType);
		switch (type) {
		case AMEX:
			cc.setCreditCardType(CreditCardTypeType.MAESTRO);
			break;
		case VISA:
			cc.setCreditCardType(CreditCardTypeType.VISA);
			break;
		case MASTERCARD:
			cc.setCreditCardType(CreditCardTypeType.MASTERCARD);
			break;
		default:
			break;
		}
		details.setPaymentDetails(paymentDetails);

		details.setCreditCard(cc);

		details.setPaymentAction(PaymentActionCodeType.fromValue("Sale"));

		pprequest.setDoDirectPaymentRequestDetails(details);
		doPaymentReq.setDoDirectPaymentRequest(pprequest);
		DoDirectPaymentResponseType resp = null;
		try {
			resp = service.doDirectPayment(doPaymentReq);
		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		if (resp != null) {
			logger.info("lastReq: " + service.getLastRequest());
			logger.info("lastResp: " + service.getLastResponse());

			checkout.setRequest(service.getLastRequest());
			checkout.setResponse(service.getLastResponse());

			if (resp.getAck().toString().equalsIgnoreCase("SUCCESS")) {
				logger.info("Ack: " + resp.getAck());
				map.put("Ack", resp.getAck());
				/*
				 * (Required) Recurring payments profile ID returned in the
				 * CreateRecurringPaymentsProfile response. Note: The profile
				 * must have a status of either Active or Suspended. Character
				 * length and limitations: 14 single-byte alphanumeric
				 * characters. 19 character profile IDs are supported for
				 * compatibility with previous versions of the PayPal API.
				 */
				map.put("Ack", resp.getAck());
				logger.info("Ack: " + resp.getAck());

				map.put("Transaction ID", resp.getTransactionID());
				logger.info("Transaction ID: " + resp.getTransactionID());

				map.put("Amount", resp.getAmount().getValue() + " "
						+ resp.getAmount().getCurrencyID());
				logger.info("Amount: " + resp.getAmount().getValue() + " "
						+ resp.getAmount().getCurrencyID());

				map.put("Payment Status", resp.getPaymentStatus());
				logger.info("Payment Status: " + resp.getPaymentStatus());
				/*
				 * Status of the recurring payment profile. ActiveProfile – The
				 * recurring payment profile has been successfully created and
				 * activated for scheduled payments according the billing
				 * instructions from the recurring payments profile.
				 * PendingProfile – The system is in the process of creating the
				 * recurring payment profile. Please check your IPN messages for
				 * an update.
				 */
				transactionService.addTransaction(checkout);
				return "secure-checkout-success";
			} else {
				logger.info("error: " + resp.getErrors());
				map.put("error", resp.getErrors());
				transactionService.addTransaction(checkout);
				return "secure-checkout-fail";
			}
		}

		logger.info("go to do-secure-checkout-success");
		return "secure-checkout-success";
	}

	@RequestMapping("/do-secure-checkout-trial")
	public String doSecureCheckoutTrial(
			@ModelAttribute("checkout") @Valid Transaction checkout,
			BindingResult result, Map<String, Object> map) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to secure-checkout");
			return "secure-checkout";
		}

		CurrencyCodeType currency = CurrencyCodeType.fromValue("GBP");
		logger.info("Currency: " + currency);
		Map<String, String> configurationMap = Configuration.getAcctAndConfig();
		PayPalAPIInterfaceServiceService service = new PayPalAPIInterfaceServiceService(
				configurationMap);

		CreateRecurringPaymentsProfileReq req = new CreateRecurringPaymentsProfileReq();
		CreateRecurringPaymentsProfileRequestType reqType = new CreateRecurringPaymentsProfileRequestType();

		/*
		 * Number of billing periods that make up one billing cycle; required if
		 * you specify an optional trial period. The combination of billing
		 * frequency and billing period must be less than or equal to one year.
		 * For example, if the billing cycle is Month, the maximum value for
		 * billing frequency is 12. Similarly, if the billing cycle is Week, the
		 * maximum value for billing frequency is 52. Note: If the billing
		 * period is SemiMonth, the billing frequency must be 1.
		 */
		int trialBillingFrequency = Integer.parseInt(checkout
				.getTrialBillingFrequency());
		logger.info("TrialBillingFrequency: " + trialBillingFrequency);

		/*
		 * Billing amount for each billing cycle during this payment period;
		 * required if you specify an optional trial period. This amount does
		 * not include shipping and tax amounts. Note: All amounts in the
		 * CreateRecurringPaymentsProfile request must have the same currency.
		 * Character length and limitations: Value is a positive number which
		 * cannot exceed $10,000 USD in any currency. It includes no currency
		 * symbol. It must have 2 decimal places, the decimal separator must be
		 * a period (.), and the optional thousands separator must be a comma
		 * (,).
		 */
		String trialBillingAmount = checkout.getTrialBillingAmount();
		logger.info("TrialBillingAmount: " + trialBillingAmount);
		BasicAmountType trialPaymentAmount = new BasicAmountType(currency,
				trialBillingAmount);
		/*
		 * Unit for billing during this subscription period; required if you
		 * specify an optional trial period. It is one of the following values:
		 * [Day, Week, SemiMonth, Month, Year] For SemiMonth, billing is done on
		 * the 1st and 15th of each month. Note: The combination of
		 * BillingPeriod and BillingFrequency cannot exceed one year.
		 */
		String trialBillingPeriod = checkout.getTrialBillingPeriod();
		logger.info("TrialBillingPeriod: " + trialBillingPeriod);
		BillingPeriodType trialPeriod = BillingPeriodType.fromValue(checkout
				.getTrialBillingPeriod());
		/*
		 * Number of billing periods that make up one billing cycle; required if
		 * you specify an optional trial period. The combination of billing
		 * frequency and billing period must be less than or equal to one year.
		 * For example, if the billing cycle is Month, the maximum value for
		 * billing frequency is 12. Similarly, if the billing cycle is Week, the
		 * maximum value for billing frequency is 52. Note: If the billing
		 * period is SemiMonth, the billing frequency must be 1.
		 */
		int trialBillingCycles = Integer.parseInt(checkout
				.getTrialBillingCycles());
		logger.info("TrialBillingCycles: " + trialBillingCycles);

		BillingPeriodDetailsType trialPeriodType = new BillingPeriodDetailsType(
				trialPeriod, trialBillingFrequency, trialPaymentAmount);
		trialPeriodType.setTotalBillingCycles(trialBillingCycles);

		/*
		 * (Optional) Tax amount for each billing cycle during this payment
		 * period. Note: All amounts in the request must have the same currency.
		 * Character length and limitations: Value is a positive number which
		 * cannot exceed $10,000 USD in any currency. It includes no currency
		 * symbol. It must have 2 decimal places, the decimal separator must be
		 * a period (.), and the optional thousands separator must be a comma
		 * (,).
		 */
		String trialTaxAmount = checkout.getTrialTaxAmount();
		logger.info("TrialTaxAmount: " + trialTaxAmount);

		trialPeriodType.setTaxAmount(new BasicAmountType(currency,
				trialTaxAmount));

		ScheduleDetailsType trialScheduleDetails = new ScheduleDetailsType();

		trialScheduleDetails.setTrialPeriod(trialPeriodType);

		/*
		 * (Required) The date when billing for this profile begins. Note: The
		 * profile may take up to 24 hours for activation. Character length and
		 * limitations: Must be a valid date, in UTC/GMT format
		 */
		String billingStartDate = checkout.getBillingStartDate();
		logger.info("BillingStartDate: " + billingStartDate);
		RecurringPaymentsProfileDetailsType profileDetails = new RecurringPaymentsProfileDetailsType(
				billingStartDate + "T00:00:00:000Z");

		/*
		 * (Required) Description of the recurring payment. Note: You must
		 * ensure that this field matches the corresponding billing agreement
		 * description included in the SetExpressCheckout request. Character
		 * length and limitations: 127 single-byte alphanumeric characters
		 */
		String profileDescription = checkout.getService();
		logger.info("ProfileDescription: " + profileDescription);
		ScheduleDetailsType scheduleDetails = new ScheduleDetailsType();

		scheduleDetails.setTrialPeriod(trialPeriodType);

		scheduleDetails.setDescription(profileDescription);

		/*
		 * (Required) Number of billing periods that make up one billing cycle.
		 * The combination of billing frequency and billing period must be less
		 * than or equal to one year. For example, if the billing cycle is
		 * Month, the maximum value for billing frequency is 12. Similarly, if
		 * the billing cycle is Week, the maximum value for billing frequency is
		 * 52. Note: If the billing period is SemiMonth, the billing frequency
		 * must be 1.
		 */
		String billingFrequency = checkout.getBillingFrequency();
		logger.info("BillingFrequency: " + billingFrequency);

		int frequency = Integer.parseInt(billingFrequency);
		/*
		 * (Required) Billing amount for each billing cycle during this payment
		 * period. This amount does not include shipping and tax amounts. Note:
		 * All amounts in the CreateRecurringPaymentsProfile request must have
		 * the same currency. Character length and limitations: Value is a
		 * positive number which cannot exceed $10,000 USD in any currency. It
		 * includes no currency symbol. It must have 2 decimal places, the
		 * decimal separator must be a period (.), and the optional thousands
		 * separator must be a comma (,).
		 */
		String billingAmount = checkout.getPrice();
		logger.info("BillingAmount: " + billingAmount);

		BasicAmountType paymentAmount = new BasicAmountType(currency,
				billingAmount);
		/*
		 * (Required) Unit for billing during this subscription period. It is
		 * one of the following values: [Day, Week, SemiMonth, Month, Year] For
		 * SemiMonth, billing is done on the 1st and 15th of each month. Note:
		 * The combination of BillingPeriod and BillingFrequency cannot exceed
		 * one year.
		 */
		String billingPeriod = checkout.getBillingPeriod();
		logger.info("BillingPeriod: " + billingPeriod);

		BillingPeriodType period = BillingPeriodType.fromValue(billingPeriod);
		logger.info("BillingPeriodType: " + period.toString());
		/*
		 * (Optional) Number of billing cycles for payment period. For the
		 * regular payment period, if no value is specified or the value is 0,
		 * the regular payment period continues until the profile is cancelled
		 * or deactivated. For the regular payment period, if the value is
		 * greater than 0, the regular payment period will expire after the
		 * trial period is finished and continue at the billing frequency for
		 * TotalBillingCycles cycles.
		 */
		String totalBillingCycles = checkout.getTotalBillingCycles();
		logger.info("TotalBillingCycles: " + totalBillingCycles);
		BillingPeriodDetailsType paymentPeriod = new BillingPeriodDetailsType(
				period, frequency, paymentAmount);
		try {
			int numCycles = Integer.parseInt(totalBillingCycles);
			paymentPeriod.setTotalBillingCycles(numCycles);
		} catch (Exception e) {
			logger.error("Problem getting the Total Billing Cycles", e);
		}

		/*
		 * (Optional) Tax amount for each billing cycle during this payment
		 * period. Note: All amounts in the request must have the same currency.
		 * Character length and limitations: Value is a positive number which
		 * cannot exceed $10,000 USD in any currency. It includes no currency
		 * symbol. It must have 2 decimal places, the decimal separator must be
		 * a period (.), and the optional thousands separator must be a comma
		 * (,).
		 */
		String taxAmount = checkout.getVat();
		logger.info("TaxAmount: " + taxAmount);

		paymentPeriod.setTaxAmount(new BasicAmountType(currency, taxAmount));

		scheduleDetails.setPaymentPeriod(paymentPeriod);

		CreateRecurringPaymentsProfileRequestDetailsType reqDetails = new CreateRecurringPaymentsProfileRequestDetailsType(
				profileDetails, scheduleDetails);
		/*
		 * credit card number is required for CreateRecurringPaymentsProfile.
		 * Each CreateRecurringPaymentsProfile request creates a single
		 * recurring payments profile. Note:
		 */
		CreditCardDetailsType cc = new CreditCardDetailsType();

		/*
		 * (Required) Credit card number. Character length and limitations:
		 * Numeric characters only with no spaces or punctuation. The string
		 * must conform with modulo and length required by each credit card
		 * type.
		 */
		String creditCardNumber = checkout.getCardNumber();
		logger.info("CreditCardNumber: " + creditCardNumber);

		cc.setCreditCardNumber(creditCardNumber);

		/*
		 * Card Verification Value, version 2. Your Merchant Account settings
		 * determine whether this field is required. To comply with credit card
		 * processing regulations, you must not store this value after a
		 * transaction has been completed. Character length and limitations: For
		 * Visa, MasterCard, and Discover, the value is exactly 3 digits. For
		 * American Express, the value is exactly 4 digits.
		 */
		String cvv = checkout.getSecurityNumber();
		logger.info("CVV: " + cvv);

		cc.setCVV2(cvv);

		String issue = checkout.getIssue();
		if (issue != null && !issue.equals("")) {
			cc.setIssueNumber(issue);
		}

		// Start Month
		String startMonth = checkout.getStartMonth();
		logger.info("StartMonth: " + startMonth);

		cc.setStartMonth(Integer.parseInt(startMonth));

		// Start Year
		String startYear = checkout.getStartYear();
		logger.info("StartYear: " + startYear);

		cc.setStartYear(Integer.parseInt(startYear));

		// Expiry Month
		String expMonth = checkout.getEndMonth();
		logger.info("ExpMonth: " + expMonth);

		cc.setExpMonth(Integer.parseInt(expMonth));

		// Expiry Year
		String expYear = checkout.getEndYear();
		logger.info("ExpYear: " + expYear);

		cc.setExpYear(Integer.parseInt(expYear));
		PayerInfoType payerInfo = new PayerInfoType();

		/*
		 * (Required) Email address of buyer. Character length and limitations:
		 * 127 single-byte characters
		 */
		String buyerEmailId = checkout.getEmail();
		logger.info("BuyerEmailId: " + buyerEmailId);

		payerInfo.setPayer(buyerEmailId);

		String forname = checkout.getForename();
		String surname = checkout.getSurname();
		String mobile = checkout.getMobile();

		payerInfo.setContactPhone(mobile);

		PersonNameType payerName = new PersonNameType();
		payerName.setFirstName(forname);
		payerName.setLastName(surname);
		payerInfo.setPayerName(payerName);
		cc.setCardOwner(payerInfo);
		/*
		 * (Optional) Type of credit card. For UK, only Maestro, MasterCard,
		 * Discover, and Visa are allowable. For Canada, only MasterCard and
		 * Visa are allowable and Interac debit cards are not supported. It is
		 * one of the following values: [ Visa, MasterCard, Discover, Amex,
		 * Maestro: See note.] Note: If the credit card type is Maestro, you
		 * must set CURRENCYCODE to GBP. In addition, you must specify either
		 * STARTDATE or ISSUENUMBER.
		 */
		String creditCardType = checkout.getCreditCardType();
		logger.info("CreditCardType: " + creditCardType);

		CreditCardTypeType type = CreditCardTypeType.fromValue(creditCardType);
		switch (type) {
		case AMEX:
			cc.setCreditCardType(CreditCardTypeType.MAESTRO);
			break;
		case VISA:
			cc.setCreditCardType(CreditCardTypeType.VISA);
			break;
		case MASTERCARD:
			cc.setCreditCardType(CreditCardTypeType.MASTERCARD);
			break;
		default:
			break;
		}

		reqDetails.setCreditCard(cc);

		reqType.setCreateRecurringPaymentsProfileRequestDetails(reqDetails);
		req.setCreateRecurringPaymentsProfileRequest(reqType);
		CreateRecurringPaymentsProfileResponseType resp = null;
		try {
			resp = service.createRecurringPaymentsProfile(req);
		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		if (resp != null) {
			logger.info("lastReq: " + service.getLastRequest());
			logger.info("lastResp: " + service.getLastResponse());

			checkout.setRequest(service.getLastRequest());
			checkout.setResponse(service.getLastResponse());

			if (resp.getAck().toString().equalsIgnoreCase("SUCCESS")) {
				logger.info("Ack: " + resp.getAck());
				map.put("Ack", resp.getAck());
				/*
				 * (Required) Recurring payments profile ID returned in the
				 * CreateRecurringPaymentsProfile response. Note: The profile
				 * must have a status of either Active or Suspended. Character
				 * length and limitations: 14 single-byte alphanumeric
				 * characters. 19 character profile IDs are supported for
				 * compatibility with previous versions of the PayPal API.
				 */
				map.put("profileID", resp
						.getCreateRecurringPaymentsProfileResponseDetails()
						.getProfileID());
				logger.info("Profile ID: "
						+ resp.getCreateRecurringPaymentsProfileResponseDetails()
								.getProfileID());

				map.put("transactionID", resp
						.getCreateRecurringPaymentsProfileResponseDetails()
						.getTransactionID());
				logger.info("Transaction ID: "
						+ resp.getCreateRecurringPaymentsProfileResponseDetails()
								.getTransactionID());
				/*
				 * Status of the recurring payment profile. ActiveProfile – The
				 * recurring payment profile has been successfully created and
				 * activated for scheduled payments according the billing
				 * instructions from the recurring payments profile.
				 * PendingProfile – The system is in the process of creating the
				 * recurring payment profile. Please check your IPN messages for
				 * an update.
				 */
				map.put("Profile Status", resp
						.getCreateRecurringPaymentsProfileResponseDetails()
						.getProfileStatus());
				logger.info("Profile Status: "
						+ resp.getCreateRecurringPaymentsProfileResponseDetails()
								.getProfileStatus());
				transactionService.addTransaction(checkout);
				return "secure-checkout-success";
			} else {
				logger.info("error: " + resp.getErrors());
				map.put("error", resp.getErrors());
				transactionService.addTransaction(checkout);
				return "secure-checkout-fail";
			}
		}

		logger.info("go to do-secure-checkout-success");
		return "secure-checkout-success";
	}
}
