package uk.co.trinitylogic.skyfall.controller;

import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;

import uk.co.trinitylogic.skyfall.form.Advice;
import uk.co.trinitylogic.skyfall.form.Member;
import uk.co.trinitylogic.skyfall.form.ReportPreview;
import uk.co.trinitylogic.skyfall.form.Service;
import uk.co.trinitylogic.skyfall.service.AdviceService;
import uk.co.trinitylogic.skyfall.service.MemberService;
import uk.co.trinitylogic.skyfall.service.ServiceService;

@Controller
public class ReportController {

	protected final Log logger = LogFactory.getLog(getClass());
	protected final String separator = System.getProperty("line.separator");
	protected final Format formatter = new SimpleDateFormat("dd/MM/yy");

	@Autowired
	private ServiceService serviceService;

	@Autowired
	private AdviceService adviceService;

	@Autowired
	private MemberService memberService;

	@RequestMapping("/list-daily-report")
	public String listDailyReports(Map<String, Object> map) {
		List<Service> services = serviceService.listLiveService();
		List<Service> reportServices = new ArrayList<Service>();
		for (Service service : services) {
			List<Advice> adv = adviceService.listDailyAdviceForService(
					service.getServiceId(), getTodaysDate());
			service.setAdvice(adv);
			reportServices.add(service);
		}
		map.put("services", reportServices);
		logger.info("go to list-daily-reports-success");
		return "list-daily-reports-success";
	}

	@RequestMapping("/preview-daily-service-report-{serviceId}")
	public String previewDailyServiceReport(@PathVariable Integer serviceId,
			Map<String, Object> map) {
		List<Advice> advices = adviceService.listDailyAdviceForService(
				serviceId, getTodaysDate());
		Service service = serviceService.getService(serviceId);
		StringBuilder mail = new StringBuilder();
		mail.append("Hi $Forename$,").append(separator).append(separator);
		mail.append("Here is today's information...").append(separator)
				.append(separator);
		mail.append(advices.size()).append(" bet(s).").append(separator)
				.append(separator);
		for (Advice advice : advices) {
			mail.append(advice.emailString()).append(separator);
		}
		StringBuilder text = new StringBuilder();
		text.append(service.getShortCode()).append(" ")
				.append(formatter.format(new Date())).append(" ");
		for (Advice advice : advices) {
			text.append(advice.textString()).append(". ");
		}
		ReportPreview reportPreview = new ReportPreview();
		reportPreview.setMail(mail.toString());
		reportPreview.setText(text.toString());
		map.put("service", service);
		map.put("reportPreview", reportPreview);
		logger.info("preview-daily-service-report-success");
		return "preview-daily-service-report-success";
	}

	@RequestMapping("/send-both-daily-service-report-{serviceId}")
	public String sendBothDailyServiceReport(@PathVariable Integer serviceId,
			Map<String, Object> map) {
		List<Advice> advices = adviceService.listDailyAdviceForService(
				serviceId, getTodaysDate());
		Service service = serviceService.getService(serviceId);
		List<Member> mailMembers = new ArrayList<Member>();
		List<Member> textMembers = new ArrayList<Member>();
		mailMembers = memberService.listMembersForMailServiceOptIn(service);
		textMembers = memberService.listMembersForTextServiceOptIn(service);
		StringBuilder mail = new StringBuilder();
		mail.append("Hi $Forename$,").append(separator).append(separator);
		mail.append("Here is today's information...").append(separator)
				.append(separator);
		for (Advice advice : advices) {
			mail.append(advice.emailString()).append(separator);
		}
		StringBuilder text = new StringBuilder();
		text.append(service.getShortCode()).append(" ")
				.append(formatter.format(new Date())).append(" ");
		for (Advice advice : advices) {
			text.append(advice.textString()).append(". ");
		}
		adviceService.sendMailAdviceForService(service, advices, mailMembers,
				mail.toString());
		adviceService.sendTextAdviceForService(service, advices, textMembers,
				text.toString());
		map.put("service", service);
		map.put("mailMembers", mailMembers);
		map.put("textMembers", textMembers);
		map.put("advices", advices);
		logger.info("send-daily-service-report-success");
		return "send-daily-service-report-success";
	}

	@RequestMapping("/send-daily-service-report-{serviceId}")
	public String sendDailyServiceReport(@PathVariable Integer serviceId,
			@ModelAttribute ReportPreview reportPreview, Map<String, Object> map) {
		List<Advice> advices = adviceService.listDailyAdviceForService(
				serviceId, getTodaysDate());
		Service service = serviceService.getService(serviceId);
		List<Member> mailMembers = new ArrayList<Member>();
		List<Member> textMembers = new ArrayList<Member>();
		if (reportPreview.isSendMail()) {
			mailMembers = memberService.listMembersForMailServiceOptIn(service);
		}
		if (reportPreview.isSendText()) {
			textMembers = memberService.listMembersForTextServiceOptIn(service);
		}
		adviceService.sendMailAdviceForService(service, advices, mailMembers,
				reportPreview.getMail());
		adviceService.sendTextAdviceForService(service, advices, textMembers,
				reportPreview.getText());
		map.put("service", service);
		map.put("mailMembers", mailMembers);
		map.put("textMembers", textMembers);
		map.put("advices", advices);
		logger.info("send-daily-service-report-success");
		return "send-daily-service-report-success";
	}

	private Date getTodaysDate() {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.set(Calendar.MILLISECOND, 0);
		return cal.getTime();
	}
}
