package uk.co.trinitylogic.skyfall.controller;

import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.validation.Valid;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import uk.co.trinitylogic.skyfall.form.Advice;
import uk.co.trinitylogic.skyfall.form.Member;
import uk.co.trinitylogic.skyfall.form.Report;
import uk.co.trinitylogic.skyfall.form.Service;
import uk.co.trinitylogic.skyfall.service.AdviceService;
import uk.co.trinitylogic.skyfall.service.MemberService;
import uk.co.trinitylogic.skyfall.service.ServiceService;

@Controller
public class ServiceController {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private ServiceService serviceService;

	@Autowired
	private MemberService memberService;

	@Autowired
	private AdviceService adviceService;

	@RequestMapping("/list-service")
	public String listService(Map<String, Object> map) {
		List<Service> serviceList = serviceService.listLiveServiceWithMemberCount();
		long memberCount = 0;
		for (Service service : serviceList) {
			memberCount = memberCount + service.getMembers();
		}
		map.put("serviceList", serviceList);
		map.put("memberCount", new Long(memberCount));
		logger.info("go to list-service-success");
		return "list-service-success";
	}

	@RequestMapping("/add-service")
	public String addService(Map<String, Object> map) {
		map.put("service", new Service());
		logger.info("go to add-service");
		return "add-service";
	}

	@RequestMapping(value = "/add-service-submit", method = RequestMethod.POST)
	public String addServiceSubmit(@ModelAttribute @Valid Service service, BindingResult result) {
		if (result.hasErrors()) {
			logger.info("result has errors go back to add-service");
			return "add-service";
		}
		serviceService.addService(service);
		logger.info("go to add-service-success");
		return "add-service-success";
	}

	@RequestMapping(value = "/delete-service-{serviceId}", method = RequestMethod.GET)
	public String deleteService(@PathVariable Integer serviceId, Map<String, Object> map) {
		serviceService.removeService(serviceId);
		map.put("service", serviceService.getService((serviceId)));
		logger.info("go to delete-service-success for serviceId: " + serviceId);
		return "delete-service-success";
	}

	@RequestMapping(value = "/hide-service-{serviceId}", method = RequestMethod.GET)
	public String hideService(@PathVariable Integer serviceId, Map<String, Object> map) {
		serviceService.hideService(serviceId);
		map.put("service", serviceService.getService((serviceId)));
		logger.info("go to hide-service-success for serviceId: " + serviceId);
		return "hide-service-success";
	}

	@RequestMapping(value = "/update-service-{serviceId}", method = RequestMethod.GET)
	public String updateService(@PathVariable Integer serviceId, Map<String, Object> map) {
		map.put("service", serviceService.getService((serviceId)));
		logger.info("go to update-service for serviceId: " + serviceId);
		return "update-service";
	}

	@RequestMapping(value = "/update-service-submit-{serviceId}", method = RequestMethod.POST)
	public String updateServiceSubmit(@PathVariable Integer serviceId, @ModelAttribute @Valid Service service, BindingResult result) {
		service.setServiceId(serviceId);
		if (result.hasErrors()) {
			logger.info("result has errors go back to update-service for serviceId: " + serviceId);
			return "update-service";
		}
		List<Member> members = memberService.listMembersForService(service);
		service.setMember(members);
		serviceService.updateService(service);
		logger.info("go to update-service-success for serviceId: " + serviceId);
		return "update-service-success";
	}

	@RequestMapping(value = "/list-service-member-{serviceId}", method = RequestMethod.GET)
	public String listServiceMember(@PathVariable Integer serviceId, Map<String, Object> map) {
		Service service = serviceService.getService(serviceId);
		List<Member> members = memberService.listMembersForService(service);
		map.put("service", service);
		map.put("members", members);
		logger.info("go to list-service-member for serviceId: " + serviceId);
		return "list-service-member";
	}

	@RequestMapping(value = "/list-service-member-full-{serviceId}", method = RequestMethod.GET)
	public String listServiceMemberFull(@PathVariable Integer serviceId, Map<String, Object> map) {
		Service service = serviceService.getService(serviceId);
		List<Member> members = memberService.listMembersForService(service);
		map.put("service", service);
		map.put("members", members);
		logger.info("go to list-service-member-full for serviceId: " + serviceId);
		return "list-service-member-full";
	}

	@RequestMapping(value = "/list-service-advice-all-{serviceId}", method = RequestMethod.GET)
	public String listServiceAdviceAll(@PathVariable Integer serviceId, Map<String, Object> map) {
		Service service = serviceService.getService(serviceId);
		List<Advice> advices = adviceService.listAllAdvicesForService(service);
		Report report = new Report(service, advices);
		report.setType("all");
		map.put("service", service);
		map.put("advices", advices);
		map.put("report", report);
		logger.info("go to list-service-advice for serviceId: " + serviceId);
		return "list-service-advice";
	}

	@RequestMapping(value = "/list-service-advice-between-{serviceId}", method = RequestMethod.GET)
	public String listServiceAdviceBetween(@PathVariable Integer serviceId, @RequestParam("startDate") @DateTimeFormat(pattern = "dd/MM/yy") Date startDate,
			@RequestParam("endDate") @DateTimeFormat(pattern = "dd/MM/yy") Date endDate, Map<String, Object> map) {
		Service service = serviceService.getService(serviceId);
		List<Advice> advices = adviceService.listAdvicesForService(service, startDate, endDate);
		Report report = new Report(service, advices);
		report.setType("between");
		map.put("service", service);
		map.put("advices", advices);
		map.put("report", report);
		logger.info("go to list-service-advice for serviceId: " + serviceId);
		System.out.println(startDate + "  |  " + endDate);
		return "list-service-advice";
	}
}
