package uk.co.trinitylogic.skyfall.dao;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import uk.co.trinitylogic.skyfall.form.Enquirer;
import uk.co.trinitylogic.skyfall.form.Member;

@Repository
@Transactional
public class EnquirerDAOImpl implements EnquirerDAO {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private SessionFactory sessionFactory;

	@Override
	public void addEnquirer(Enquirer enquirer) {
		logger.info("Adding a new enquirer: " + enquirer);
		long startTime = System.nanoTime();
		sessionFactory.getCurrentSession().save(enquirer);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Added a new enquirer: " + enquirer + " Time: " + duration
				+ "ms");
	}

	@Override
	public boolean hasEnquired(Enquirer enquirer) {
		logger.info("Check if: " + enquirer + " is a new enquirer");
		long startTime = System.nanoTime();
		String hql = "select count(*) from Enquirer enq where enq.email =:email";
		Long result = (Long) sessionFactory.getCurrentSession()
				.createQuery(hql).setParameter("email", enquirer.getEmail())
				.uniqueResult();
		boolean hasEnquired = result > 0;
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Check if: " + enquirer + " is a new enquirer took Time: "
				+ duration + "ms");
		return hasEnquired;
	}

	@Override
	public Enquirer getEnquirer(Integer id) {
		logger.info("Get enquirer for enquirerId: " + id);
		long startTime = System.nanoTime();
		Enquirer enquirer = (Enquirer) sessionFactory.getCurrentSession().get(
				Enquirer.class, id);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Get enquirer for enquirerId: " + id + " Time: " + duration
				+ "ms");
		return enquirer;
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Enquirer> getEnquirer(String email) {
		logger.info("Get enquirer for email: " + email);
		long startTime = System.nanoTime();
		String hql = "select enq from Enquirer enq where enq.email =:email";
		List<Enquirer> result = sessionFactory.getCurrentSession()
				.createQuery(hql).setParameter("email", email)
				.setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Get enquirer for email: " + email + " Time: " + duration
				+ "ms");
		return result;
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Enquirer> listEnquirer() {
		logger.info("List all enquirers");
		String hql = "select enquirer from Enquirer enquirer order by enquirer.created desc";
		long startTime = System.nanoTime();
		List<Enquirer> result = sessionFactory.getCurrentSession()
				.createQuery(hql).setCacheable(true).list();
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List all enquirers Time: " + duration + "ms");
		return result;
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Enquirer> listUnsubscribes() {
		logger.info("List all Unsubscribes");
		String hql = "select enquirer from Enquirer enquirer order by enquirer.created desc";
		long startTime = System.nanoTime();
		List<Enquirer> enquirers = sessionFactory.getCurrentSession()
				.createQuery(hql).setCacheable(true).list();
		List<Enquirer> result = new ArrayList<Enquirer>();
		for (Enquirer e : enquirers) {
			String email = e.getEmail();
			logger.info("Get member for email: " + email);
			String sql = "select mem from Member mem join mem.service ser where mem.email1 =:email";
			List<Member> member = sessionFactory.getCurrentSession()
					.createQuery(sql).setParameter("email", email).list();
			if (member.size() > 0) {
				Member mem = member.get(0);
				if (mem.getDoubleOptIn() == false) {
					logger.info("Got member for email: " + email
							+ " and they have Un-Subscribed");
					result.add(e);
				}
			}
		}
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("List all Unsubscribes Time: " + duration + "ms");
		return result;
	}

	@Override
	public void updateEnquirer(Enquirer enquirer) {
		logger.info("Update enquirerId: " + enquirer);
		long startTime = System.nanoTime();
		sessionFactory.getCurrentSession().update(enquirer);
		long endTime = System.nanoTime();
		long duration = (endTime - startTime) / 1000000;
		logger.info("Update an enquirer Time: " + duration + "ms");
	}

}
