package uk.co.trinitylogic.skyfall.form;

import java.text.Format;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.persistence.Cacheable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.lang.WordUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;
import org.hibernate.search.annotations.Analyzer;
import org.hibernate.search.annotations.DocumentId;
import org.hibernate.search.annotations.Field;
import org.hibernate.search.annotations.Index;
import org.hibernate.search.annotations.Indexed;
import org.hibernate.search.annotations.Store;
import org.springframework.format.annotation.DateTimeFormat;

@Entity
@Indexed(index = "advices")
@Analyzer(impl = StandardAnalyzer.class)
@Table(name = "advices")
@Cacheable
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class Advice {

	@Transient
	protected final Log logger = LogFactory.getLog(getClass());

	@Id
	@GeneratedValue
	@Column(name = "idadvices")
	@DocumentId
	private Integer adviceId;

	@Column(name = "created")
	@DateTimeFormat(pattern = "dd/MM/yy")
	private Date created = new Date();

	@Column(nullable = true, name = "course")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String course;

	@Column(nullable = true, name = "time")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String time;

	@Column(nullable = true, name = "bet")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String bet;

	@Column(nullable = true, name = "bet_type")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String betType;

	@Column(nullable = true, name = "best_early_price_ep")
	private String bestEarlyPriceEp;

	@Column(nullable = true, name = "best_ep_bookie_exchange")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String bestEpBookieExchange;

	@Column(nullable = true, name = "betfair_starting_price_bsp")
	private String betfairStartingPriceBsp;

	@Column(nullable = true, name = "finishing_position")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String finishingPosition;

	@Column(nullable = true, name = "won_or_lost")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String wonOrLost;

	@Column(nullable = true, name = "ls_at_ep")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String lsAtEp;

	@Column(nullable = true, name = "lsp_at_bsp")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String lspAtBsp;

	@Column(nullable = true, name = "recorded_stake")
	private String recordedStake;

	@Column(nullable = true, name = "racecard_number")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String racecardNumber;

	@Column(nullable = true, name = "market")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String market;

	@Column(nullable = true, name = "type")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String type;

	@Column(nullable = true, name = "recommended_stake")
	private String recommendedStake;

	@ManyToOne
	@NotFound(action = NotFoundAction.IGNORE)
	@JoinColumn(name = "services_idservices")
	@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
	private Service service;

	public Integer getAdviceId() {
		return adviceId;
	}

	public void setAdviceId(Integer adviceId) {
		this.adviceId = adviceId;
	}

	public String getCourse() {
		return course;
	}

	public void setCourse(String course) {
		this.course = course;
	}

	public String getTime() {
		return time;
	}

	public Date getTimeAsDate() {
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
		Date startTime;
		try {
			startTime = sdf.parse(time);
		} catch (ParseException e) {
			startTime = new Date();
		}
		return startTime;
	}

	public void setTime(String time) {
		this.time = time;
	}

	public String getBet() {
		return bet;
	}

	public void setBet(String bet) {
		this.bet = bet;
	}

	public String getBetType() {
		return betType;
	}

	public void setBetType(String betType) {
		this.betType = betType;
	}

	public String getBestEarlyPriceEp() {
		return bestEarlyPriceEp;
	}

	public void setBestEarlyPriceEp(String bestEarlyPriceEp) {
		this.bestEarlyPriceEp = bestEarlyPriceEp;
	}

	public String getBestEpBookieExchange() {
		return bestEpBookieExchange;
	}

	public void setBestEpBookieExchange(String besEpBookieExchange) {
		this.bestEpBookieExchange = besEpBookieExchange;
	}

	public String getBetfairStartingPriceBsp() {
		return betfairStartingPriceBsp;
	}

	public void setBetfairStartingPriceBsp(String betfairStartingPriceBsp) {
		this.betfairStartingPriceBsp = betfairStartingPriceBsp;
	}

	public String getFinishingPosition() {
		return finishingPosition;
	}

	public void setFinishingPosition(String finishingPosition) {
		this.finishingPosition = finishingPosition;
	}

	public String getWonOrLost() {
		return wonOrLost;
	}

	public void setWonOrLost(String wonOrLost) {
		this.wonOrLost = wonOrLost;
	}

	public String getLsAtEp() {
		return lsAtEp;
	}

	public void setLsAtEp(String lsAtEp) {
		this.lsAtEp = lsAtEp;
	}

	public String getLspAtBsp() {
		return lspAtBsp;
	}

	public void setLspAtBsp(String lspAtBsp) {
		this.lspAtBsp = lspAtBsp;
	}

	public Service getService() {
		return service;
	}

	public void setService(Service service) {
		this.service = service;
	}

	public String getRecordedStake() {
		return recordedStake;
	}

	public void setRecordedStake(String recordedStake) {
		this.recordedStake = recordedStake;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public Date getCreated() {
		return created;
	}

	public String getRacecardNumber() {
		return racecardNumber;
	}

	public void setRacecardNumber(String racecardNumber) {
		this.racecardNumber = racecardNumber;
	}

	public String getMarket() {
		return market;
	}

	public void setMarket(String market) {
		this.market = market;
	}

	public String getRecommendedStake() {
		return recommendedStake;
	}

	public void setRecommendedStake(String recommendedStake) {
		this.recommendedStake = recommendedStake;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getFormattedCreated() {
		Format formatter = new SimpleDateFormat("dd/MM/yy");
		if (created == null) {
			created = new Date();
		}
		return formatter.format(created);
	}

	@Override
	public String toString() {
		return "[" + getFormattedCreated() + "] [" + service.getShortCode() + "] [" + WordUtils.capitalizeFully(course) + "] [" + time + "] [" + WordUtils.capitalizeFully(bet) + "] [" + betType
				+ "] [" + racecardNumber + "] [" + bestEarlyPriceEp + "] [" + bestEpBookieExchange + "]";
	}

	public String emailString() {
		if ("RACING".equalsIgnoreCase(type)) {
			return WordUtils.capitalizeFully(course) + " " + time + " #" + racecardNumber + " " + WordUtils.capitalizeFully(bet) + ("WIN".equalsIgnoreCase(betType) ? "" : " " + betType) + " "
					+ bestEarlyPriceEp + " " + bestEpBookieExchange;
		}
		if ("SPORTS".equalsIgnoreCase(type)) {
			return WordUtils.capitalizeFully(course) + " " + recommendedStake + " " + WordUtils.capitalizeFully(bet) + ("WIN".equalsIgnoreCase(betType) ? "" : " " + betType) + " " + bestEarlyPriceEp
					+ " " + bestEpBookieExchange;
		}
		return "";
	}

	public String getDetails() {
		return emailString();
	}

	public String textString() {
		String shortCourse = WordUtils.capitalizeFully(course.substring(0, 3));
		if ("RACING".equalsIgnoreCase(type)) {
			return shortCourse + " " + time + " #" + racecardNumber + " " + WordUtils.capitalizeFully(bet) + ("WIN".equalsIgnoreCase(betType) ? "" : " " + betType) + " " + bestEarlyPriceEp + " "
					+ bestEpBookieExchange;
		}
		if ("SPORTS".equalsIgnoreCase(type)) {
			return shortCourse + " " + recommendedStake + " " + WordUtils.capitalizeFully(bet) + ("WIN".equalsIgnoreCase(betType) ? "" : " " + betType) + " " + bestEarlyPriceEp + " "
					+ bestEpBookieExchange;
		}
		return "";
	}
}
