package uk.co.trinitylogic.skyfall.form;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.Cacheable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;
import org.hibernate.search.annotations.Analyzer;
import org.hibernate.search.annotations.DocumentId;
import org.hibernate.search.annotations.Field;
import org.hibernate.search.annotations.Index;
import org.hibernate.search.annotations.Indexed;
import org.hibernate.search.annotations.Store;
import org.hibernate.validator.constraints.Email;
import org.springframework.format.annotation.DateTimeFormat;

@Entity
@Indexed(index = "members")
@Analyzer(impl = StandardAnalyzer.class)
@Table(name = "members")
@Cacheable
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class Member {

	@Transient
	protected final Log logger = LogFactory.getLog(getClass());

	@Id
	@Column(name = "idmembers")
	@GeneratedValue
	@DocumentId
	private Integer memberId;

	@Column(name = "created")
	private Date created = new Date();

	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	@Column(nullable = true, name = "title")
	private String title;

	@Column(nullable = true, name = "forename")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String forename;

	@Column(nullable = true, name = "middlename")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String middlename;

	// @NotEmpty
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	@Column(nullable = true, name = "surname")
	private String surname;

	@Column(nullable = true, name = "address_1")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String address1;

	@Column(nullable = true, name = "address_2")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String address2;

	@Column(nullable = true, name = "town")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String town;

	@Column(nullable = true, name = "county")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String county;

	@Column(nullable = true, name = "postcode")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String postcode;

	@Column(nullable = true, name = "country")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String country;

	@Column(nullable = true, name = "hilite")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String hilite;

	@Column(nullable = true, name = "hilite_date_cleaned")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	@DateTimeFormat(pattern = "dd/MM/yy")
	private Date hiliteDateCleaned;

	// @Pattern(regexp = "[0-9]{12}", message =
	// "Please enter 12 digit mobile number i.e 441234567890")
	@Column(nullable = true, name = "mobile")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String mobile;

	@Column(nullable = true, name = "mobile_opt_in")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private Boolean mobileOptIn;

	@Column(nullable = true, name = "mobile_opt_in_for_bets")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private Boolean mobileOptInForBets;

	// @Pattern(regexp = "[0-9]{11}", message =
	// "Please enter 11 digit phone number i.e 01234123456")
	@Column(nullable = true, name = "telephone_home")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String telephoneHome;

	@Column(nullable = true, name = "telephone_work")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String telephoneWork;

	@Email
	@Column(nullable = true, name = "email_1")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String email1 = "";

	@Column(nullable = true, name = "email_1_opt_in")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private Boolean email1OptIn;

	@Column(nullable = true, name = "email_1_opt_in_bets")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private Boolean email1OptInBets;

	@Column(nullable = true, name = "email_2")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private String email2;

	@Column(nullable = true, name = "email_2_opt_in")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private Boolean email2OptIn;

	@Column(nullable = true, name = "postal_opt_in")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private Boolean postalOptIn;

	@Column(nullable = true, name = "double_opt_in")
	@Field(index = Index.UN_TOKENIZED, store = Store.NO)
	private Boolean doubleOptIn;

	@ManyToMany
	@NotFound(action = NotFoundAction.IGNORE)
	@JoinTable(name = "members_has_services", joinColumns = { @JoinColumn(name = "members_idmembers") }, inverseJoinColumns = { @JoinColumn(name = "services_idservices") })
	@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
	private Set<Service> service;

	public Integer getMemberId() {
		return memberId;
	}

	public void setMemberId(Integer memberId) {
		this.memberId = memberId;
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public String getForename() {
		return forename;
	}

	public void setForename(String forename) {
		this.forename = forename;
	}

	public String getMiddlename() {
		return middlename;
	}

	public void setMiddlename(String middlename) {
		this.middlename = middlename;
	}

	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	public String getAddress1() {
		return address1;
	}

	public void setAddress1(String address1) {
		this.address1 = address1;
	}

	public String getAddress2() {
		return address2;
	}

	public void setAddress2(String address2) {
		this.address2 = address2;
	}

	public String getTown() {
		return town;
	}

	public void setTown(String town) {
		this.town = town;
	}

	public String getCounty() {
		return county;
	}

	public void setCounty(String county) {
		this.county = county;
	}

	public String getPostcode() {
		return postcode;
	}

	public void setPostcode(String postcode) {
		this.postcode = postcode;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public String getHilite() {
		return hilite;
	}

	public void setHilite(String hilite) {
		this.hilite = hilite;
	}

	public Date getHiliteDateCleaned() {
		return hiliteDateCleaned;
	}

	public void setHiliteDateCleaned(Date hiliteDateCleaned) {
		this.hiliteDateCleaned = hiliteDateCleaned;
	}

	public String getMobile() {
		return mobile;
	}

	public void setMobile(String mobile) {
		this.mobile = mobile;
	}

	public Boolean getMobileOptIn() {
		return mobileOptIn;
	}

	public void setMobileOptIn(Boolean mobileOptIn) {
		this.mobileOptIn = mobileOptIn;
	}

	public Boolean getMobileOptInForBets() {
		return mobileOptInForBets;
	}

	public void setMobileOptInForBets(Boolean mobileOptInForBets) {
		this.mobileOptInForBets = mobileOptInForBets;
	}

	public String getTelephoneHome() {
		return telephoneHome;
	}

	public void setTelephoneHome(String telephoneHome) {
		this.telephoneHome = telephoneHome;
	}

	public String getTelephoneWork() {
		return telephoneWork;
	}

	public void setTelephoneWork(String telephoneWork) {
		this.telephoneWork = telephoneWork;
	}

	public String getEmail1() {
		return email1;
	}

	public void setEmail1(String email1) {
		this.email1 = email1;
	}

	public Boolean getEmail1OptIn() {
		return email1OptIn;
	}

	public void setEmail1OptIn(Boolean email1OptIn) {
		this.email1OptIn = email1OptIn;
	}

	public Boolean getEmail1OptInBets() {
		return email1OptInBets;
	}

	public void setEmail1OptInBets(Boolean email1OptInBets) {
		this.email1OptInBets = email1OptInBets;
	}

	public String getEmail2() {
		return email2;
	}

	public void setEmail2(String email2) {
		this.email2 = email2;
	}

	public Boolean getEmail2OptIn() {
		return email2OptIn;
	}

	public void setEmail2OptIn(Boolean email2OptIn) {
		this.email2OptIn = email2OptIn;
	}

	public Set<Service> getService() {
		return service;
	}

	public void setService(Set<Service> service) {
		this.service = service;
	}

	public void removeService(Service ser) {
		if (service != null) {
			service.remove(ser);
		}
	}

	public void addService(Service ser) {
		if (service == null) {
			service = new HashSet<Service>();
		}
		service.add(ser);
	}

	public Date getCreated() {
		return created;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public Boolean getPostalOptIn() {
		return postalOptIn;
	}

	public void setPostalOptIn(Boolean postalOptIn) {
		this.postalOptIn = postalOptIn;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("[");
		for (Service s : service) {
			sb.append(s.getShortCode()).append(" | ");
		}
		if (sb.length() > 3) {
			sb.delete(sb.length() - 3, sb.length());
		}
		sb.append("]");
		String result = title + " " + forename + " " + surname + " " + mobile + " " + email1 + " " + sb.toString();
		return result;
	}

	public String toCsvString() {
		StringBuilder sb = new StringBuilder();
		sb.append("[");
		if (service != null) {
			for (Service s : service) {
				sb.append(s.getShortCode()).append(" | ");
			}
			if (sb.length() > 3) {
				sb.delete(sb.length() - 3, sb.length());
			}
			sb.append("]");
		}
		String result = memberId + "," + created + "," + title + "," + forename + "," + middlename + "," + surname + "," + address1 + "," + address2 + "," + town + "," + county + "," + postcode + ","
				+ country + "," + hilite + "," + mobile + "," + mobileOptIn + "," + mobileOptInForBets + "," + telephoneHome + "," + telephoneWork + "," + email1 + "," + email1OptIn + ","
				+ email1OptInBets + "," + email2 + "," + email2OptIn + "," + postalOptIn + "," + sb.toString();
		return result;
	}

	public String getListServices() {
		StringBuilder sb = new StringBuilder();
		sb.append("[");
		int size = service.size();
		int i = 1;
		for (Service s : service) {
			if (size != i) {
				sb.append(s.getServiceId()).append("|");
			} else {
				sb.append(s.getServiceId()).append("]");
			}
			i++;
		}
		return sb.toString();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((this.memberId == null) ? 0 : memberId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		}
		if (obj == null || obj.getClass() != this.getClass()) {
			return false;
		}
		Member member = (Member) obj;
		return this.memberId.intValue() == member.memberId.intValue();
	}

	public Boolean getDoubleOptIn() {
		return doubleOptIn;
	}

	public void setDoubleOptIn(Boolean doubleOptIn) {
		this.doubleOptIn = doubleOptIn;
	}
}
