package uk.co.trinitylogic.skyfall.form;

import java.util.Date;
import java.util.List;

import javax.persistence.Cacheable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.annotations.NotFound;
import org.hibernate.annotations.NotFoundAction;
import org.hibernate.validator.constraints.NotEmpty;

@Entity
@Table(name = "services")
@Cacheable
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class Service {

	@Transient
	protected final Log logger = LogFactory.getLog(getClass());

	@Id
	@Column(name = "idservices")
	@GeneratedValue
	private Integer serviceId;

	@Column(name = "created")
	private Date created = new Date();

	@NotEmpty
	@Column(nullable = false, name = "name")
	private String name = "";

	@NotEmpty
	@Column(nullable = false, name = "description")
	private String description = "";

	@NotEmpty
	@Column(nullable = false, name = "short_code")
	private String shortCode = "";

	@Column(nullable = true, name = "live")
	private Boolean live = true;

	@Column(nullable = false, name = "hidden")
	private Boolean hidden = false;

	@OneToMany(mappedBy = "service")
	@NotFound(action = NotFoundAction.IGNORE)
	private List<Advice> advice;

	@ManyToMany
	@NotFound(action = NotFoundAction.IGNORE)
	@JoinTable(name = "members_has_services", joinColumns = { @JoinColumn(name = "services_idservices") }, inverseJoinColumns = { @JoinColumn(name = "members_idmembers") })
	private List<Member> member;

	@Transient
	private Long members;

	public Integer getServiceId() {
		return serviceId;
	}

	public void setServiceId(Integer serviceId) {
		this.serviceId = serviceId;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public List<Advice> getAdvice() {
		return advice;
	}

	public void setAdvice(List<Advice> advice) {
		this.advice = advice;
	}

	public String getShortCode() {
		return shortCode;
	}

	public void setShortCode(String shortCode) {
		this.shortCode = shortCode;
	}

	public List<Member> getMember() {
		return member;
	}

	public void setMember(List<Member> member) {
		this.member = member;
	}

	public Date getCreated() {
		return created;
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public Boolean getLive() {
		return live;
	}

	public void setLive(Boolean live) {
		this.live = live;
	}

	@Override
	public String toString() {
		return shortCode;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((this.serviceId == null) ? 0 : serviceId.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == this) {
			return true;
		}
		if (obj == null || obj.getClass() != this.getClass()) {
			return false;
		}
		Service service = (Service) obj;
		return this.serviceId == service.serviceId;
	}

	public Long getMembers() {
		return members;
	}

	public void setMembers(Long members) {
		this.members = members;
	}

	public void setHidden(Boolean hidden) {
		this.hidden = hidden;
	}

	public Boolean getHidden() {
		return hidden;
	}
}
