package uk.co.trinitylogic.skyfall.form;

import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.persistence.Cacheable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;
import org.hibernate.validator.constraints.CreditCardNumber;
import org.hibernate.validator.constraints.Email;
import org.hibernate.validator.constraints.NotEmpty;
import org.springframework.format.annotation.DateTimeFormat;

@Entity
@Table(name = "transactions")
@Cacheable
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class Transaction {

	@Id
	@Column(name = "id")
	@GeneratedValue
	private Integer id;

	@Column(name = "created")
	@DateTimeFormat(pattern = "dd/MM/yy")
	private Date created = new Date();

	@NotEmpty(message = "Price is mandatory.")
	@Column(name = "price")
	private String price;

	@NotEmpty(message = "Total is mandatory.")
	@Column(name = "total")
	private String total;

	@NotEmpty(message = "Vat is mandatory.")
	@Column(name = "vat")
	private String vat;

	@NotEmpty(message = "Service is mandatory.")
	@Column(name = "service")
	private String service;

	@NotEmpty(message = "First Name is mandatory.")
	@Column(name = "forename")
	private String forename;

	@NotEmpty(message = "Surname is mandatory.")
	@Column(name = "surname")
	private String surname;

	@NotEmpty(message = "Address is mandatory.")
	@Column(name = "address")
	private String address;

	@NotEmpty(message = "City is mandatory.")
	@Column(name = "city")
	private String city;

	@NotEmpty(message = "County is mandatory.")
	@Column(name = "state")
	private String state;

	@NotEmpty(message = "Country is mandatory.")
	@Column(name = "country")
	private String country;

	@NotEmpty(message = "Postcode is mandatory.")
	@Column(name = "postcode")
	private String postcode;

	@Email(message = "Please enter a valid email address.")
	@Column(name = "email")
	private String email;

	@Column(name = "mobile")
	private String mobile;

	@CreditCardNumber
	@Column(name = "card_number")
	private String cardNumber;

	@NotEmpty(message = "CVV is mandatory.")
	@Column(name = "security_number")
	private String securityNumber;

	@Column(name = "issue")
	private String issue;

	@NotEmpty(message = "Start Month is mandatory.")
	@Column(name = "start_month")
	private String startMonth;

	@NotEmpty(message = "Starrt Year is mandatory.")
	@Column(name = "start_year")
	private String startYear;

	@NotEmpty(message = "End Month is mandatory.")
	@Column(name = "end_month")
	private String endMonth;

	@NotEmpty(message = "End Year is mandatory.")
	@Column(name = "end_year")
	private String endYear;

	@Column(name = "billing_start_date")
	private String billingStartDate;

	@Column(name = "billing_frequency")
	private String billingFrequency;

	@Column(name = "billing_period")
	private String billingPeriod;

	@Column(name = "total_billing_cycles")
	private String totalBillingCycles;

	@NotEmpty(message = "Card Type is mandatory.")
	@Column(name = "credit_card_type")
	private String creditCardType;

	@Column(name = "request")
	private String request;

	@Column(name = "response")
	private String response;

	@Column(name = "trial_billing_amount")
	private String trialBillingAmount;

	@Column(name = "trial_total_amount")
	private String trialTotalAmount;

	public String getTrialBillingAmount() {
		return trialBillingAmount;
	}

	public String getTrialBillingFrequency() {
		return trialBillingFrequency;
	}

	public String getTrialBillingPeriod() {
		return trialBillingPeriod;
	}

	public String getTrialBillingCycles() {
		return trialBillingCycles;
	}

	public String getTrialTaxAmount() {
		return trialTaxAmount;
	}

	@Column(name = "trial_billing_frequency")
	private String trialBillingFrequency;

	@Column(name = "trial_billing_period")
	private String trialBillingPeriod;

	@Column(name = "trial_billing_cycles")
	private String trialBillingCycles;

	@Column(name = "trial_tax_amount")
	private String trialTaxAmount;

	public String getPrice() {
		return price;
	}

	public void setPrice(String price) {
		this.price = price;
	}

	public String getVat() {
		return vat;
	}

	public void setVat(String vat) {
		this.vat = vat;
	}

	public String getService() {
		return service;
	}

	public void setService(String service) {
		this.service = service;
	}

	public String getForename() {
		return forename;
	}

	public void setForename(String forename) {
		this.forename = forename;
	}

	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	public String getAddress() {
		return address;
	}

	public void setAddress(String address) {
		this.address = address;
	}

	public String getCity() {
		return city;
	}

	public void setCity(String city) {
		this.city = city;
	}

	public String getState() {
		return state;
	}

	public void setState(String state) {
		this.state = state;
	}

	public String getCountry() {
		return country;
	}

	public void setCountry(String country) {
		this.country = country;
	}

	public String getPostcode() {
		return postcode;
	}

	public void setPostcode(String postcode) {
		this.postcode = postcode;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getMobile() {
		return mobile;
	}

	public void setMobile(String mobile) {
		this.mobile = mobile;
	}

	public Date getCreated() {
		return created;
	}

	public String getFormattedCreated() {
		Format formatter = new SimpleDateFormat("dd/MM/yy hh:mm:ss");
		if (created == null) {
			created = new Date();
		}
		return formatter.format(created);
	}

	public void setCreated(Date created) {
		this.created = created;
	}

	public String getCardNumber() {
		return cardNumber;
	}

	public void setCardNumber(String cardNumber) {
		this.cardNumber = cardNumber;
	}

	public String getSecurityNumber() {
		return securityNumber;
	}

	public void setSecurityNumber(String securityNumber) {
		this.securityNumber = securityNumber;
	}

	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	public String getTotal() {
		return total;
	}

	public void setTotal(String total) {
		this.total = total;
	}

	public String getIssue() {
		return issue;
	}

	public void setIssue(String issue) {
		this.issue = issue;
	}

	public String getStartMonth() {
		return startMonth;
	}

	public void setStartMonth(String startMonth) {
		this.startMonth = startMonth;
	}

	public String getStartYear() {
		return startYear;
	}

	public void setStartYear(String startYear) {
		this.startYear = startYear;
	}

	public String getEndMonth() {
		return endMonth;
	}

	public void setEndMonth(String endMonth) {
		this.endMonth = endMonth;
	}

	public String getEndYear() {
		return endYear;
	}

	public void setEndYear(String endYear) {
		this.endYear = endYear;
	}

	public String getBillingStartDate() {
		return billingStartDate;
	}

	public void setBillingStartDate(String billingStartDate) {
		this.billingStartDate = billingStartDate;
	}

	public String getBillingFrequency() {
		return billingFrequency;
	}

	public void setBillingFrequency(String billingFrequency) {
		this.billingFrequency = billingFrequency;
	}

	public String getCreditCardType() {
		return creditCardType;
	}

	public void setCreditCardType(String creditCardType) {
		this.creditCardType = creditCardType;
	}

	public String getRequest() {
		return request;
	}

	public void setRequest(String request) {
		this.request = request;
	}

	public String getResponse() {
		return response;
	}

	public void setResponse(String response) {
		this.response = response;
	}

	public String getBillingPeriod() {
		return billingPeriod;
	}

	public void setBillingPeriod(String billingPeriod) {
		this.billingPeriod = billingPeriod;
	}

	public String getTotalBillingCycles() {
		return totalBillingCycles;
	}

	public void setTotalBillingCycles(String totalBillingCycles) {
		this.totalBillingCycles = totalBillingCycles;
	}

	public void setTrialBillingAmount(String trialBillingAmount) {
		this.trialBillingAmount = trialBillingAmount;
	}

	public void setTrialBillingFrequency(String trialBillingFrequency) {
		this.trialBillingFrequency = trialBillingFrequency;
	}

	public void setTrialBillingPeriod(String trialBillingPeriod) {
		this.trialBillingPeriod = trialBillingPeriod;
	}

	public void setTrialBillingCycles(String trialBillingCycles) {
		this.trialBillingCycles = trialBillingCycles;
	}

	public void setTrialTaxAmount(String trialTaxAmount) {
		this.trialTaxAmount = trialTaxAmount;
	}

	public String getTrialTotalAmount() {
		return trialTotalAmount;
	}

	public void setTrialTotalAmount(String trialTotalAmount) {
		this.trialTotalAmount = trialTotalAmount;
	}

}
