package uk.co.trinitylogic.skyfall.service;

import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import uk.co.trinitylogic.skyfall.dao.AdviceDAO;
import uk.co.trinitylogic.skyfall.form.Advice;
import uk.co.trinitylogic.skyfall.form.Member;

@Service
public class AdviceServiceImpl implements AdviceService {

	protected final Log logger = LogFactory.getLog(getClass());
	protected final Format formatter = new SimpleDateFormat("dd/MM/yy");

	@Autowired
	private AdviceDAO adviceDAO;

	@Autowired
	private TxtService txtService;

	@Autowired
	private MailService mailService;

	@Override
	@Transactional
	public void addAdvice(Advice advice) {
		adviceDAO.addAdvice(advice);
	}

	@Override
	@Transactional
	public void removeAdvice(Integer id) {
		adviceDAO.removeAdvice(id);
	}

	@Override
	@Transactional
	public void updateAdvice(Advice advice) {
		adviceDAO.updateAdvice(advice);
	}

	@Override
	@Transactional
	public List<Advice> listAdvice() {
		return adviceDAO.listAdvice();
	}

	@Override
	@Transactional
	public Advice getAdvice(Integer adviceId) {
		return adviceDAO.getAdvice(adviceId);
	}

	@Override
	@Transactional
	public List<Advice> listDailyAdviceForService(Integer serviceId, Date today) {
		return adviceDAO.listDailyAdviceForService(serviceId, today);
	}

	@Override
	public void sendMailAdviceForService(
			uk.co.trinitylogic.skyfall.form.Service service,
			List<Advice> advices, List<Member> mailMembers, String mail) {
		String subject = service.getName() + " " + formatter.format(new Date());
		for (Member member : mailMembers) {
			String msg = mail.toString();
			if (member.getForename() != null) {
				msg = msg.replaceAll("\\$Forename\\$", member.getForename());
			} else {
				msg = msg.replaceAll("\\$Forename\\$", "member");
			}
			mailService.sendMail("matthew@matthewwalton.co.uk",
					member.getEmail1(), subject, msg);
		}
	}

	@Override
	public void sendTextAdviceForService(
			uk.co.trinitylogic.skyfall.form.Service service,
			List<Advice> advices, List<Member> textMembers, String text) {
		for (Member member : textMembers) {
			txtService.sendTxt(member.getMobile(), text.toString());
		}
	}

	@Override
	public List<Advice> searchForAdvices(String search)
			throws InterruptedException {
		return adviceDAO.searchForAdvices(search);
	}

	@Override
	public List<Advice> listAdvicesForService(
			uk.co.trinitylogic.skyfall.form.Service service, Date startDate,
			Date endDate) {
		return adviceDAO.listAdvicesForService(service, startDate, endDate);
	}

	@Override
	public List<Advice> listAllAdvicesForService(
			uk.co.trinitylogic.skyfall.form.Service service) {
		return adviceDAO.listAllAdvicesForService(service);
	}
}
