package uk.co.trinitylogic.skyfall.service;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

@Service
public class MailServiceImpl implements MailService {

	protected final Log logger = LogFactory.getLog(getClass());

	@Autowired
	private JavaMailSender mailSender;

	@Override
	@Async
	public void sendMail(String from, String to, String subject, String body) {
		System.setProperty("mail.mime.charset", "utf8");
		this.sendMail(from, to, null, subject, body);
	}

	@Override
	@Async
	public void sendMail(String from, String to, String bcc, String subject, String body) {
		System.setProperty("mail.mime.charset", "utf8");
		SimpleMailMessage message = new SimpleMailMessage();
		message.setFrom(from);
		message.setTo(to);
		if (bcc != null) {
			message.setBcc(bcc);
		}
		message.setSubject(subject);
		message.setText(addMailFooter(body));
		logger.info(message.toString());
		mailSender.send(message);
	}

	@Override
	@Async
	public void sendHtmlMail(String from, String to, String subject, String body) {
		System.setProperty("mail.mime.charset", "utf8");
		this.sendHtmlMail(from, to, null, subject, body);
	}

	@Override
	@Async
	public void sendHtmlMail(String from, String to, String bcc, String subject, String body) {
		System.setProperty("mail.mime.charset", "utf8");
		try {
			MimeMessage msg = mailSender.createMimeMessage();
			msg.setHeader("Content-Type", "text/html; charset=UTF-8");
			msg.setSubject(subject, "UTF-8");
			MimeMessageHelper message = new MimeMessageHelper(msg, true);
			message.setFrom(from);
			message.setTo(to);
			if (bcc != null) {
				message.setBcc(bcc);
			}
			message.setText(body, true);
			logger.info(message.toString());
			mailSender.send(msg);
		} catch (MessagingException e) {
			logger.error(e);
		}
	}

	@Override
	@Async
	public void sendMail(SimpleMailMessage message) {
		System.setProperty("mail.mime.charset", "utf8");
		logger.info(message.toString());
		mailSender.send(message);
	}

	@Override
	@Async
	public void sendMail(SimpleMailMessage... message) {
		System.setProperty("mail.mime.charset", "utf8");
		for (SimpleMailMessage msg : message) {
			logger.info(msg.toString());
		}
		mailSender.send(message);
	}

	private String addMailFooter(String body) {
		return body + getMailFooter();
	}

	public String getMailFooter() {
		StringBuffer sb = new StringBuffer();
		String sep = System.getProperty("line.separator");
		sb.append(sep).append(sep).append(sep);
		sb.append("Kind Regards,").append(sep);
		sb.append(sep);
		sb.append("Matthew").append(sep);
		sb.append(sep);
		sb.append("---------------------------").append(sep);
		sb.append("01625 315654 (office)").append(sep);
		sb.append("07752 768094 (personal mobile)").append(sep);
		sb.append("matthew@matthewwalton.co.uk").append(sep);
		sb.append("---------------------------").append(sep);
		return sb.toString();
	}

	public String getHtmlMailFooter() {
		StringBuffer sb = new StringBuffer();
		sb.append("<p></p>");
		sb.append("<hr align=\"left\" width=\"100%\" color=\"#000000\" size=\"1\">");
		sb.append("01625 315654 (office)<br>");
		sb.append("07752 768094 (personal mobile)<br>");
		sb.append("<a href=\"mailto:matthew@matthewwalton.co.uk\">matthew@matthewwalton.co.uk</a><br>");
		sb.append("<hr align=\"left\" width=\"100%\" color=\"#000000\" size=\"1\">");
		sb.append("</body></html>");
		return sb.toString();
	}
}
