package uk.co.trinitylogic.skyfall.service;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.WordUtils;
import org.junit.Test;

public class TestParsers {

	@Test
	public void parseAbDataGivenToSj060215() throws Exception {

		String path = "C:\\Users\\jacksosa\\workspace\\Skyfall\\src\\test\\resources\\ab-data-given-to-sj-060215.csv";
		File file = new File(path);
		BufferedReader br = new BufferedReader(new FileReader(file));
		List<Record> records = new ArrayList<Record>();
		String line = null;
		while ((line = br.readLine()) != null) {
			List<String> details = Arrays.asList(line.split(","));
			// System.out.println(details);
			int i = 0;
			Record record = new Record();
			record.setOrig(line);
			boolean add = false;
			for (String detail : details) {
				// System.out.println("[" + i + "] " + clean);
				if (i == 0) {
					if (detail.equals("")) {
						record.setTitle("Mr");
					} else {
						record.setTitle(cleanupWord(detail));
					}
				}
				if (i == 1) {
					record.setForename(cleanupWord(detail));
				}
				if (i == 2) {
					record.setSurname(cleanupWord(detail));
				}

				if (i == 5) {
					record.setAddress1(cleanupWord(detail));
				}

				if (i == 6) {
					record.setAddress2(cleanupWord(detail));
				}

				if (i == 7) {
					record.setTown(cleanupWord(detail));
				}

				if (i == 8) {
					record.setCounty(cleanupWord(detail));
				}

				if (i == 9) {
					record.setCountry(cleanupWord(detail));
				}

				String num = detail.replaceAll(" ", "");
				try {
					Long number = new Long(num);
					num = number.toString();
					if (!num.startsWith("0")) {
						num = "0" + num;
					}
					if (isValidTelephone(num)) {
						record.setTelephone(num);
					}

					if (isValidMoblie(num)) {
						record.setMobile(num);
					}

				} catch (Exception ex) {
				}

				if (detail.contains("@") && isValidEmailAddress(detail)) {
					record.setEmail(detail);
				}

				if (detail.length() >= 6 && detail.length() <= 8) {
					String tmp = cleanupPostcode(detail);
					if (isValidPostcode(tmp)) {
						record.setPostcode(tmp);
						add = true;
					}
				}

				i++;
			}
			String tmp = cleanupPostcode(record.getCountry());
			if (tmp.equalsIgnoreCase(record.getPostcode())) {
				record.setCountry("");
			}
			if (add) {
				records.add(record);
			}

		}
		int j = 0;
		for (Record record : records) {
			System.out.println(j++ + "," + record.toCsvString());
			// System.out.println(record.getOrig());
		}
		br.close();
	}

	public boolean isValidEmailAddress(String email) {
		String pattern = "^[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@((\\[[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\])|(([a-zA-Z\\-0-9]+\\.)+[a-zA-Z]{2,}))$";
		java.util.regex.Pattern p = java.util.regex.Pattern.compile(pattern);
		java.util.regex.Matcher m = p.matcher(email);
		return m.matches();
	}

	public boolean isValidPostcode(String postcode) {
		String pattern = "^([A-PR-UWYZ](([0-9](([0-9]|[A-HJKSTUW])?)?)|([A-HK-Y][0-9]([0-9]|[ABEHMNPRVWXY])?)) [0-9][ABD-HJLNP-UW-Z]{2})";
		java.util.regex.Pattern p = java.util.regex.Pattern.compile(pattern);
		java.util.regex.Matcher m = p.matcher(postcode);
		return m.matches();
	}

	private boolean isValidTelephone(String telephone) {
		String pattern = "^(?:0044|\\+44|0)[1-6][0-9]{9}$";
		java.util.regex.Pattern p = java.util.regex.Pattern.compile(pattern);
		java.util.regex.Matcher m = p.matcher(telephone);
		return m.matches();
	}

	private boolean isValidMoblie(String mobile) {
		String pattern = "^(?:0044|\\+44|0)[7-9][0-9]{9}$";
		java.util.regex.Pattern p = java.util.regex.Pattern.compile(pattern);
		java.util.regex.Matcher m = p.matcher(mobile);
		return m.matches();
	}

	public String cleanupPostcode(String postcode) {
		if (postcode != null) {
			StringBuilder sb = new StringBuilder(postcode.toUpperCase().replace(" ", "").trim());
			if (sb.length() == 6) {
				sb.insert(3, " ");
			} else if (sb.length() == 7) {
				sb.insert(4, " ");
			}
			return sb.toString();
		} else {
			return "";
		}
	}

	public String cleanupWord(String word) {
		if (word == null) {
			word = "";
		}
		String clean = WordUtils.capitalize(word.trim().toLowerCase());
		clean = clean.replaceAll("\"", "=");
		clean = clean.replaceAll(",", "=");

		if (clean.contains("'")) {
			String last = clean.substring(clean.indexOf("'") + 1, clean.length());
			last = WordUtils.capitalize(last);
			clean = clean.substring(0, clean.indexOf("'")) + "'" + last;
		}
		return clean;
	}
}
